/*******************************************************************************
* File Name: BAT_LEVEL.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "BAT_LEVEL.h"

static BAT_LEVEL_BACKUP_STRUCT  BAT_LEVEL_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: BAT_LEVEL_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet BAT_LEVEL_SUT.c usage_BAT_LEVEL_Sleep_Wakeup
*******************************************************************************/
void BAT_LEVEL_Sleep(void)
{
    #if defined(BAT_LEVEL__PC)
        BAT_LEVEL_backup.pcState = BAT_LEVEL_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            BAT_LEVEL_backup.usbState = BAT_LEVEL_CR1_REG;
            BAT_LEVEL_USB_POWER_REG |= BAT_LEVEL_USBIO_ENTER_SLEEP;
            BAT_LEVEL_CR1_REG &= BAT_LEVEL_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(BAT_LEVEL__SIO)
        BAT_LEVEL_backup.sioState = BAT_LEVEL_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        BAT_LEVEL_SIO_REG &= (uint32)(~BAT_LEVEL_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: BAT_LEVEL_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to BAT_LEVEL_Sleep() for an example usage.
*******************************************************************************/
void BAT_LEVEL_Wakeup(void)
{
    #if defined(BAT_LEVEL__PC)
        BAT_LEVEL_PC = BAT_LEVEL_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            BAT_LEVEL_USB_POWER_REG &= BAT_LEVEL_USBIO_EXIT_SLEEP_PH1;
            BAT_LEVEL_CR1_REG = BAT_LEVEL_backup.usbState;
            BAT_LEVEL_USB_POWER_REG &= BAT_LEVEL_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(BAT_LEVEL__SIO)
        BAT_LEVEL_SIO_REG = BAT_LEVEL_backup.sioState;
    #endif
}


/* [] END OF FILE */
