/*******************************************************************************
* File Name: FRAM_Ena.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "FRAM_Ena.h"

static FRAM_Ena_BACKUP_STRUCT  FRAM_Ena_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: FRAM_Ena_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet FRAM_Ena_SUT.c usage_FRAM_Ena_Sleep_Wakeup
*******************************************************************************/
void FRAM_Ena_Sleep(void)
{
    #if defined(FRAM_Ena__PC)
        FRAM_Ena_backup.pcState = FRAM_Ena_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            FRAM_Ena_backup.usbState = FRAM_Ena_CR1_REG;
            FRAM_Ena_USB_POWER_REG |= FRAM_Ena_USBIO_ENTER_SLEEP;
            FRAM_Ena_CR1_REG &= FRAM_Ena_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(FRAM_Ena__SIO)
        FRAM_Ena_backup.sioState = FRAM_Ena_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        FRAM_Ena_SIO_REG &= (uint32)(~FRAM_Ena_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: FRAM_Ena_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to FRAM_Ena_Sleep() for an example usage.
*******************************************************************************/
void FRAM_Ena_Wakeup(void)
{
    #if defined(FRAM_Ena__PC)
        FRAM_Ena_PC = FRAM_Ena_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            FRAM_Ena_USB_POWER_REG &= FRAM_Ena_USBIO_EXIT_SLEEP_PH1;
            FRAM_Ena_CR1_REG = FRAM_Ena_backup.usbState;
            FRAM_Ena_USB_POWER_REG &= FRAM_Ena_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(FRAM_Ena__SIO)
        FRAM_Ena_SIO_REG = FRAM_Ena_backup.sioState;
    #endif
}


/* [] END OF FILE */
