/*******************************************************************************
* File Name: TEC_Ctrl.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "TEC_Ctrl.h"

static TEC_Ctrl_BACKUP_STRUCT  TEC_Ctrl_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: TEC_Ctrl_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet TEC_Ctrl_SUT.c usage_TEC_Ctrl_Sleep_Wakeup
*******************************************************************************/
void TEC_Ctrl_Sleep(void)
{
    #if defined(TEC_Ctrl__PC)
        TEC_Ctrl_backup.pcState = TEC_Ctrl_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            TEC_Ctrl_backup.usbState = TEC_Ctrl_CR1_REG;
            TEC_Ctrl_USB_POWER_REG |= TEC_Ctrl_USBIO_ENTER_SLEEP;
            TEC_Ctrl_CR1_REG &= TEC_Ctrl_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(TEC_Ctrl__SIO)
        TEC_Ctrl_backup.sioState = TEC_Ctrl_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        TEC_Ctrl_SIO_REG &= (uint32)(~TEC_Ctrl_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: TEC_Ctrl_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to TEC_Ctrl_Sleep() for an example usage.
*******************************************************************************/
void TEC_Ctrl_Wakeup(void)
{
    #if defined(TEC_Ctrl__PC)
        TEC_Ctrl_PC = TEC_Ctrl_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            TEC_Ctrl_USB_POWER_REG &= TEC_Ctrl_USBIO_EXIT_SLEEP_PH1;
            TEC_Ctrl_CR1_REG = TEC_Ctrl_backup.usbState;
            TEC_Ctrl_USB_POWER_REG &= TEC_Ctrl_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(TEC_Ctrl__SIO)
        TEC_Ctrl_SIO_REG = TEC_Ctrl_backup.sioState;
    #endif
}


/* [] END OF FILE */
