/*******************************************************************************
* File Name: TEC_Current.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "TEC_Current.h"

static TEC_Current_BACKUP_STRUCT  TEC_Current_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: TEC_Current_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet TEC_Current_SUT.c usage_TEC_Current_Sleep_Wakeup
*******************************************************************************/
void TEC_Current_Sleep(void)
{
    #if defined(TEC_Current__PC)
        TEC_Current_backup.pcState = TEC_Current_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            TEC_Current_backup.usbState = TEC_Current_CR1_REG;
            TEC_Current_USB_POWER_REG |= TEC_Current_USBIO_ENTER_SLEEP;
            TEC_Current_CR1_REG &= TEC_Current_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(TEC_Current__SIO)
        TEC_Current_backup.sioState = TEC_Current_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        TEC_Current_SIO_REG &= (uint32)(~TEC_Current_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: TEC_Current_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to TEC_Current_Sleep() for an example usage.
*******************************************************************************/
void TEC_Current_Wakeup(void)
{
    #if defined(TEC_Current__PC)
        TEC_Current_PC = TEC_Current_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            TEC_Current_USB_POWER_REG &= TEC_Current_USBIO_EXIT_SLEEP_PH1;
            TEC_Current_CR1_REG = TEC_Current_backup.usbState;
            TEC_Current_USB_POWER_REG &= TEC_Current_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(TEC_Current__SIO)
        TEC_Current_SIO_REG = TEC_Current_backup.sioState;
    #endif
}


/* [] END OF FILE */
