/*******************************************************************************
* File Name: TEC_PWM_PM.c
* Version 3.30
*
* Description:
*  This file provides the power management source code to API for the
*  PWM.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "TEC_PWM.h"

static TEC_PWM_backupStruct TEC_PWM_backup;


/*******************************************************************************
* Function Name: TEC_PWM_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  TEC_PWM_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void TEC_PWM_SaveConfig(void) 
{

    #if(!TEC_PWM_UsingFixedFunction)
        #if(!TEC_PWM_PWMModeIsCenterAligned)
            TEC_PWM_backup.PWMPeriod = TEC_PWM_ReadPeriod();
        #endif /* (!TEC_PWM_PWMModeIsCenterAligned) */
        TEC_PWM_backup.PWMUdb = TEC_PWM_ReadCounter();
        #if (TEC_PWM_UseStatus)
            TEC_PWM_backup.InterruptMaskValue = TEC_PWM_STATUS_MASK;
        #endif /* (TEC_PWM_UseStatus) */

        #if(TEC_PWM_DeadBandMode == TEC_PWM__B_PWM__DBM_256_CLOCKS || \
            TEC_PWM_DeadBandMode == TEC_PWM__B_PWM__DBM_2_4_CLOCKS)
            TEC_PWM_backup.PWMdeadBandValue = TEC_PWM_ReadDeadTime();
        #endif /*  deadband count is either 2-4 clocks or 256 clocks */

        #if(TEC_PWM_KillModeMinTime)
             TEC_PWM_backup.PWMKillCounterPeriod = TEC_PWM_ReadKillTime();
        #endif /* (TEC_PWM_KillModeMinTime) */

        #if(TEC_PWM_UseControl)
            TEC_PWM_backup.PWMControlRegister = TEC_PWM_ReadControlRegister();
        #endif /* (TEC_PWM_UseControl) */
    #endif  /* (!TEC_PWM_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: TEC_PWM_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration of the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  TEC_PWM_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void TEC_PWM_RestoreConfig(void) 
{
        #if(!TEC_PWM_UsingFixedFunction)
            #if(!TEC_PWM_PWMModeIsCenterAligned)
                TEC_PWM_WritePeriod(TEC_PWM_backup.PWMPeriod);
            #endif /* (!TEC_PWM_PWMModeIsCenterAligned) */

            TEC_PWM_WriteCounter(TEC_PWM_backup.PWMUdb);

            #if (TEC_PWM_UseStatus)
                TEC_PWM_STATUS_MASK = TEC_PWM_backup.InterruptMaskValue;
            #endif /* (TEC_PWM_UseStatus) */

            #if(TEC_PWM_DeadBandMode == TEC_PWM__B_PWM__DBM_256_CLOCKS || \
                TEC_PWM_DeadBandMode == TEC_PWM__B_PWM__DBM_2_4_CLOCKS)
                TEC_PWM_WriteDeadTime(TEC_PWM_backup.PWMdeadBandValue);
            #endif /* deadband count is either 2-4 clocks or 256 clocks */

            #if(TEC_PWM_KillModeMinTime)
                TEC_PWM_WriteKillTime(TEC_PWM_backup.PWMKillCounterPeriod);
            #endif /* (TEC_PWM_KillModeMinTime) */

            #if(TEC_PWM_UseControl)
                TEC_PWM_WriteControlRegister(TEC_PWM_backup.PWMControlRegister);
            #endif /* (TEC_PWM_UseControl) */
        #endif  /* (!TEC_PWM_UsingFixedFunction) */
    }


/*******************************************************************************
* Function Name: TEC_PWM_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves the user configuration. Should be called
*  just prior to entering sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  TEC_PWM_backup.PWMEnableState:  Is modified depending on the enable
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void TEC_PWM_Sleep(void) 
{
    #if(TEC_PWM_UseControl)
        if(TEC_PWM_CTRL_ENABLE == (TEC_PWM_CONTROL & TEC_PWM_CTRL_ENABLE))
        {
            /*Component is enabled */
            TEC_PWM_backup.PWMEnableState = 1u;
        }
        else
        {
            /* Component is disabled */
            TEC_PWM_backup.PWMEnableState = 0u;
        }
    #endif /* (TEC_PWM_UseControl) */

    /* Stop component */
    TEC_PWM_Stop();

    /* Save registers configuration */
    TEC_PWM_SaveConfig();
}


/*******************************************************************************
* Function Name: TEC_PWM_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration. Should be called just after
*  awaking from sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  TEC_PWM_backup.pwmEnable:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void TEC_PWM_Wakeup(void) 
{
     /* Restore registers values */
    TEC_PWM_RestoreConfig();

    if(TEC_PWM_backup.PWMEnableState != 0u)
    {
        /* Enable component's operation */
        TEC_PWM_Enable();
    } /* Do nothing if component's block was disabled before */

}


/* [] END OF FILE */
