/**
  ******************************************************************************
  * @file    	adc.c
  * @brief   	Control ADC
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#include  "adc.h"
#include  "algorithm.h"
#include  <stdio.h>

adc_sample_t  adcSample;
log_reading_t logSample;  
float V0;
float Q0;
char dataString[256];
/**
  * @brief  Convert ADC readings to meaningful numbers like voltage & current
  * @param  adcSample - pointer to data read back from ADC
  * @param  logSample - pointer to data to get logged
  * @retval 0 on success
  */
uint32_t convert_adc_results( adc_sample_t * adcSample, log_reading_t *logSample){
  logSample->tHot = convert_adc_temperature( adcSample->vTemp1);
  logSample->tCold = convert_adc_temperature( adcSample->vTemp2);
  logSample->timestamp = now();
  logSample->miFan = fan_current( adcSample->iFan);
  logSample->miTec = tec_current( adcSample->iTec);
  logSample->mvTec = tec_voltage( );
  sprintf( dataString, "ADC: Hot: %04d Cold: %04d fan: %04d vTec: %04d iTec: %04d \n\r",
    (int)logSample->tHot, (int)logSample->tCold, (int)logSample->miFan, (int)logSample->miTec, (int)logSample->mvTec);
  UART1_UartPutString(dataString);
  logSample->tSkin = calc_skin_temp( adcSample->vBatt, adcSample->vTemp1, adcSample->vTemp2);
  Q0 = calcQzero( V0, logSample->tHot, logSample->tCold);
  logSample->COP   = calcCOP( Q0, logSample->mvTec,  logSample->miTec);
  logSample->mvBatt = batt_voltage( adcSample->vBatt);
  //  Monitor battery voltage...
  if( logSample->mvBatt  < CRIT_THRESHOLD){
    processEvent( CE_V_Critical);
  }
  else if( logSample->mvBatt < LOW_THRESHOLD){
    processEvent( CE_V_Low);
  }
  return 0;
}

/**
  * @brief  Read the ADC registers after sample complete
  * @param  adcSample - pointer to values to save
  * @retval 0 on success
  */

uint32_t read_adc( adc_sample_t * adcSample){
  uint32_t status;
  uint32_t attempts = 100;
  TEC_ADC_StartConvert();
  
  do{
    status = TEC_ADC_IsEndConversion(TEC_ADC_WAIT_FOR_RESULT);
    if( attempts){
      attempts--;
    }
    else{
      break;
    }
  }while (!status);
  if( status){
    adcSample->vTemp2 = TEC_ADC_GetResult16(0u);
    adcSample->vTemp1 = TEC_ADC_GetResult16(1u);
    adcSample->iFan   = TEC_ADC_GetResult16(2u);
    adcSample->iTec   = TEC_ADC_GetResult16(3u);
    adcSample->vBatt  = TEC_ADC_GetResult16(4u);
    
    convert_adc_results( adcSample, &logSample);
    return 0;
  }
  else{
    log_error("Failed to read ADC properly.");
    return 1;
  }
}

/*  Comments from the hardware design document...
The thermistors used follow Equation 1 to calculate resistance value. A look up table is use to correlate the resistance with temperature.  
R = (A / (4095-A))*100,000 Ohms; A = ADC count				(1)
The TEC current is calculated using the Equation 2.
	I = ((A / 4095)*(3.3/50))/0.025 A						(2)
The Fan current is calculated using the Equation 3.
	I = ((A / 4095)*(3.3/50))/0.51 A						(3)

*/



/**
  * @brief  Calculate battery voltage based on ADC reading
  * @param  ADC reading
  * @retval battery voltage in mV
  * @note   Ryan simplified the conversion to simply multiplying by 3.3
  *         to get the divider voltage, then by 3 again to get the 
  *         battery voltage. Last I checked, 3.3 * 3 is 9.9
  *
  */
float  batt_voltage( int16_t vBatt){
  float32 temp = (float32)vBatt/(float32)ADC_RESOLUTION;
  float32 v = 9.9 * (float32)temp;  //  Ratio of ADC voltage
    if( vBatt > ADC_RESOLUTION){
    return 999;
  }
  v *= 1000;
  return( int32_t) v;
}
/**
  * @note Following table entries have a resistance part and a temperature part. The resistance part is scaled up by a factor of 1000
  *       to allow integer operations 
  */
adc_conversion_t adc_lookup[NUM_COEFFICIENTS]={
/* R*1000     T(C)  */
  {2522,     125},  
  {2916,     120},  
  {3380,     115},  
  {3934,     110},  
  {4594,     105},  
  {5384,     100},  
  {6337,     95},
  {7481,     90},  
  {8873,     85},  
  {10566,    80},  
  {12635,    75},  
  {15184,    70},  
  {18323,    65},  
  {22224,    60},
  {27091,    55},  
  {33195,    50},  
  {40904,    45},  
  {50677,    40},  
  {63167,    35},  
  {79222,    30},  
  {100000,   25},  
  {127057,   20},
  {162559,   15},  
  {209489,   10},  
  {272011,   5}, 
  {355975,   0},  
  {469678,   -5},  
  {624987,   -10},  
  {839019,   -15},  
  {1136690,  -20},
  {1554600,  -25},  
  {2147000,  -30},  
  {2995040,  -35},  
  {4221280,  -40}
};

/**
  * @brief  Convert ADC reading from thermistors into a temperature
  * @param  Input ADC reading (0..4095)
  * @retval Celsius temperature * 10 (e.g. answer of 251 is 25.1C)
  * @note   The thermistors used follow Equation 1 to calculate resistance value. 
  * @note   A look up table is use to correlate the resistance with temperature.  
  * @note   R = (A / (4095-A))*100,000 Ohms; A = ADC count				(1)
  *
  */
float convert_adc_temperature( uint32_t vADC){
  int64_t Res, temp, interp_len, interp_point;
  float32 interp_ratio, interp_temp;
  
  if( vADC > ADC_RESOLUTION){
    return 999;
  }
  temp = (int64_t)ADC_RESOLUTION - (int64_t)vADC;
  Res  = (int64_t)vADC * 1000000;   //  Scale up * 1000
  Res /= temp;
  /**
    * @note Scan through the table of coefficients (starts high and decreases)
    *       until we overflow the table or find a coefficient less than our
    *       sample.
    *       Interpolate between points[i] and [i-1]:
    *       Say, we have a sample of 725.85
    *         {624.987,   -10}, <-- (adc_lookup[i-1])
    *           725.85  <-- point in the middle
    *         {839.019,   -15}, <-- (adc_lookup[i])
    *   interp_len is the distance between two ADC points ( here, 839 - 624 = 215)
    *   interp_point distance between the low ADC limit and the ADC reading (here 839 - 725 = 114
    *   So, the ADC is (114/215) between the 624 and 839 readings. The temperature 
    *   will be (114/215) between -10 and -15 (-15 - -10 = -5). -5 * (114/215) = -2.65
    *   -10 + -2.65 = -12.65
    *
    * Ryan's equation for R
    *   R = (A / (4095-A))*100,000 Ohms; A = ADC count
    *
    */
  for( int i = 0; i < NUM_COEFFICIENTS; i++){
    if( Res < adc_lookup[i].adcPoint){    //  lookup[i-1] <= Res <= lookup[i]
      
      interp_len = adc_lookup[i].adcPoint - adc_lookup[i-1].adcPoint;
      interp_point = Res - adc_lookup[i-1].adcPoint;
      interp_ratio = (float32)interp_point/(float32)interp_len;
      
      //  We now have the percentage of the distance in ADC counts.
      //  Now convert that percentage to degrees..
      //  Say we are between these two points at ADC = 9000:
      //  {8873,     85},  10566-8873=1693
      //  {10566,    80},  9000-8873=127
      //  percentage is 127/1693=0.0750
      //  distance in degrees is adc_lookup[i].tempPoint - adc_lookup[i-1].tempPoint = 5 (the table calibration)
      //  So, 80 - 85 = -5. -5 * .0750 = -0.375
      //  85 - .375 = 84.625
      interp_temp = adc_lookup[i-1].tempPoint + ( -5.0 * interp_ratio);
      interp_temp *= 10.0;
      
      return (int32_t) interp_temp;
    }
  }
  return 999;
}
/**
  * @brief  Return TEC voltage
  * @param  None
  * @retval TEC voltage in mV
  * @note   Calculated based on duty cycle of the PWM
  */
float  tec_voltage( void){
  int8_t TEC_PWM = TEC_PWM_ReadCompare();
  
  float32 v = (float32)TEC_PWM / (float32)ADC_RESOLUTION;
  v *= 3.3;
  v *= 1000.0;  //  mV
  return v;
}
/**
  * @brief  Convert fan's ADC reading into current in mA
  * @param  vFan is ADC voltage
  * @retval Fan current in mA
  * @note   The Fan current is calculated using the Equation 3.
  *       	I = ((A / 4095)*(3.3/50))/0.51 A						(3)
  */
float  fan_current( uint16_t iFan){
  float32 temp = (float32)iFan/(float32)ADC_RESOLUTION;
  temp *= 3.3;
  temp /= 50.0;
  temp /= 0.51;
  temp *= 1000.0;
  return (int32_t) temp;
}

/**
  * @brief  Convert TEC's ADC reading into current in mA
  * @param  vTec is ADC voltage
  * @retval TEC current in mA
  * @note The TEC current is calculated using the Equation 2.
  *     	I = ((A / 4095)*(3.3/50))/0.025 A						(2)
  */

float  tec_current( uint16_t iTec){
  float32 temp = (float32)iTec/(float32)ADC_RESOLUTION;
  temp *= 3.3;
  temp /= 50.0;
  temp /= 0.025;
  temp *= 1000.0; //  mA
  return temp;
}
/**
  * @brief  Calculate skin temperature
  * @param  tHot is hot side temp on TEC
  * @param  tCold is the cold side temp on TEC
  * @retval Skin temperature * 10 to allow .1C resolution
  * @note   Q is calculated from an equation Ryan will supply
  * @note   Rliner and Rtec are thermal resistance values
  *         that need to be supplied
  * @note   K is a constant that may or may not be needed
  */
float  calc_skin_temp( float V0, float tHot, float tCold){
  float tSk;
  float Q0 =  calcQzero( V0, tHot, tCold);
  switch( getState()){
    case  ST_COOL:
      tSk= tHot + (rLiner * Q0 );
      return tSk;
      break;
    default:
      tSk = tCold+(rLiner*((tCold-tHot)/rTec))+K;
      return tSk;
      break;
  }
  return 997.0;
}


/* [] END OF FILE */
