/**
  ******************************************************************************
  * @file    	algorithm.c
  * @brief   	Control voltage calculations
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#include "algorithm.h"
  

float tSetpoint;          //  Skin temperature we want to achieve
int16_t coolDuration;     //  How many seconds of cooling before refractory period? (allows 18 hours, which is WAY over need)
int16_t refractDuration;  //  How many seconds of refractory before cooling period again?


float coeff1 = COEFF1_DEFAULT;
float coeff2 = COEFF2_DEFAULT;
float coeff3 = COEFF3_DEFAULT;
float coeff4 = COEFF4_DEFAULT;
float coeff5 = COEFF5_DEFAULT;
float coeff6 = COEFF6_DEFAULT;
float coeff7 = COEFF7_DEFAULT;

/**
  * @brief  Calculate instantaneous heat removed (Q0)
  * @param  V0 - voltage at the TEC
  * @param  Th - temperature of TEC hot side
  * @param  Tl - temperature of TEC cold side
  * @retval Q0 - heat removed
  */
float calcQzero( float V0, float Th, float Tl){
  float Q0;
  float dT = Th - Tl;
//  Q0 = 3.63 * V0;
//  Q0 += ( Th * 0.089);
//  Q0 -= dT * 0.979;
//  Q0 -= 0.776;
  Q0 = coeff1 * V0;
  Q0 += coeff2 * Th;
  Q0 -= coeff3 * dT;
  Q0 -= coeff4;
  
  return Q0;
}
/**
  * @brief  Calculate new control voltage to apply to TEC 
  * @param  Qset - setpoint for heat removal
  * @param  Th - temperature of TEC hot side
  * @param  Tl - temperature of TEC cold side
  * @retval Vnew - TEC control voltage
  */
float calcVnew( float Qset, float Th, float Tl){
  float Vnew;
  float dT = Th - Tl;
  
//  Vnew = Qset/3.63;
//  Vnew += dT * 0.0270;
//  Vnew -= Th * 0.0245;
//  Vnew += 0.214;
  Vnew = Qset / coeff1;
  Vnew += coeff5 * dT;
  Vnew -= coeff6 * Th;
  Vnew += coeff7;
  return Vnew;
}
/**
  * @brief  Calculate system efficiency
  * @param  Q0 - heat being removed
  * @param  Vtec - voltage applied to TEC
  * @param  Itec - current into TEC
  * @retval COP
  * @note   Efficiency of the system is (not surprisingly)
  *         heat removed (e.g. work accomplished) 
  *         divided by I * V (energy supplied to system)
  * COP=Q/W=Heat Removed/(Voltage to TEC*Current to TEC)
  */
float calcCOP( float Q0, float Vtec, float Itec){
  float COP;
  float W = Vtec * Itec;
  COP = Q0/W;
  return COP;
}
/**
  * @brief  Test Q0 function with test vectors
  * @param  None
  * @retval Pass (0) Fail (number of errors)
  */
/**
  * @note Q0 test vectors
  * Q	V	DT	Th
  */
Q0_vector_t tv[] = {
  {17.47,	4.6,	0.0,	30.0},
  {8.73,	2.7,	3.0,	30.0},
  {9.31,	3.4,	5.0,	30.0},
  {2.18,	2.0,	7.0,	30.0},
  {11.21,	5.0,	9.0,	30.0}
};


uint32_t  Q0TestIterator( void){
  float Q0;
  float Tl;
  float error;
  uint32_t  result = 0; //  Success so far!
  uint32_t elements = sizeof( tv) / sizeof( Q0_vector_t);
  
  for( uint32_t i = 0; i < elements; i++){
    Tl = tv[ i].Th - tv[ i].DT;
    Q0 = calcQzero( tv[ i].V, tv[ i].Th, Tl);
    error = Q0 - tv[ i].Q;
    if( error < 0.0){
      error *= -1.0;
    }
    if( (error / Q0) > 0.01){
      result++;
    }
  }
  return result;
}
/* [] END OF FILE */
