/**
  ******************************************************************************
  * @file    	algorithm.h
  * @brief   	Control voltage calculations
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
  #ifndef __ALGORITHM_H_
  #define __ALGORITHM_H_
  #include "project.h"

extern float tSetpoint;
extern int16_t coolDuration;
extern int16_t refractDuration;

  /**
    * @note   This block of code calculates the control voltage applied
    *         to the TEC to achieve a given set point
    *   Q0      is the heat removed now 
    *   Qset    desired heat to remove
    *   V0      current voltage to TEC
    *   Vnew    new voltage to apply - this is calculate
    *   Th      Hot side temperature
    *   Tl      Low side temperature
    *   dT      Th - Tl
    *   
    *   According to Dr Myers, 
    *     Q0 = 3.63 * V0 + 0.089 * Th - 0.979 * dT - 0.776
    * and
    *     Vnew = Qset/3.63 + 0.0270 * dT - 0.0245 * Th + 0.214
    *
    */
/**
  * @note These equations are based on data collected and analyzed by Dr Myers, and they are taken
  * verbatim from my white board where he wrote them down for me...
  *
  * All the numbers need names, so 
  *
  * Q0 - 3.63V0 + 0.089Th - 0.979dT - 0.776
  *        |         |         |         |
  *        |         |         |         +-Coeff4
  *        |         |         +-----------Coeff3
  *        |         +---------------------Coeff2
  *        +-------------------------------Coeff1
  *        
  *
  * Vnew = Qset/3.63 + 0.0270dT - 0.0245Th + 0.214
  *              |        |         |         +-Coeff7
  *              |        |         +-----------Coeff6
  *              |        +---------------------Coeff5
  *              +------------------------------Coeff1 (again, right?)
  *
  */
#define COEFF1_DEFAULT    3.63
#define COEFF2_DEFAULT    0.089
#define COEFF3_DEFAULT    0.979
#define COEFF4_DEFAULT    0.776
#define COEFF5_DEFAULT    0.270
#define COEFF6_DEFAULT    0.0245
#define COEFF7_DEFAULT    0.214

/**
  * @note   This structure is used to test the calculations
  *         In algorithm.c, this structure is used to create a table
  *         of values and expected results from Dr. Myers.
  */
typedef struct{
    float Q;
    float V;
    float DT;
    float Th;
}Q0_vector_t;

float calcQzero( float V0, float Th, float Tl);
float calcVnew( float Qset, float Th, float Tl);
float calcCOP( float Q0, float Vtec, float Itec);
uint32_t  Q0TestIterator( void);

#endif  //  __ALGORITHM_H_
  
/* [] END OF FILE */
