/**
  ******************************************************************************
  * @file    	comms.h
  * @brief   	Control communications between device and PC
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#include  "comms.h"
#include  "fram.h"
#include  "algorithm.h"
#include  "rtc.h"

/**
  * @brief  Routine to read FRAM and push data out the UART
  * @param  none
  * @retval Success of the operation
  */
uint32_t  dumpLogFile( void){
  fram_read_fram( FRAM_ADDR);
  return 0;
}

/**
  * @brief  Routine to clear the log file, which is erasing the FRAM
  * @param  none
  * @retval success of the operation
  */
uint32_t  clearLogFile( void){
  fram_erase_fram( FRAM_ADDR);
  return 0;
}

/**
  * @brief  Routine to get system parameters and save them
  * @param  none
  * @retval Success of the operation
  */
uint32_t  configSystemVariables( void){
  return 0;
}

/**
  * @brief  - Routine to program the RTC
  * @param  - none
  * @retval - success of the operation
*/
uint32_t  configSystemTime( void){
  return 0;
}
/**
  * @brief  - Routine to read sys vars back toPC
  * @param  - none
  * @retval - success of the operation
  */
uint32_t  readSystemVariables( void){
  return 0;
}

/**
  * @brief  - routing to read system time back to PC
  * @param  - none
  * @retval - success of operation
  */
uint32_t  readSystemTime( void){
  return 0;
}

/**
  * @brief  - Routine to interpret serial commands
  * @param  - In - first character in the serial buffer
  *           (that's how we know to do this)
  * @retval - success of the operation
  */
uint8_t rxInput[ 100];

int32_t interpret( uint8_t in){
  static  uint8_t sequenceNumber = 0;
  
  uint32_t inPtr;
  uint8_t  inChr = in;
  uint8_t  checkSum;
  uint16_t  errorBits;
  rx_states_t chrState;
  uint8_t   msgSize;
  
  if( HEADERCHAR == inChr){
    chrState = RX_START;
    checkSum = 0;
    errorBits = 0;
    inPtr = 0;
    /**
      * @note - Got the right start, so let's vet the message
      */
    do{
      checkSum ^= inChr;
      switch( chrState){
        case RX_START:
          chrState = RX_VERSION;
          rxInput[ inPtr++] = inChr;
          break;
        case RX_VERSION:
          chrState = RX_SEQUENCE;
          rxInput[ inPtr++] = inChr;
          if( VERSIONCHAR != inChr){
            errorBits |= VERSION_ERROR;
          }
          break;
        case RX_SEQUENCE:
          chrState = RX_COUNT;
          rxInput[ inPtr++] = inChr;
          if( ( inChr - 1) == sequenceNumber){
            sequenceNumber++;
          }
          else{
            errorBits |= SEQUENCE_ERROR;
          }
          break;
        case RX_COUNT:
          rxInput[ inPtr++] = inChr;
          msgSize = inChr;
          chrState = RX_DATA;
          break;
        case RX_DATA:
          rxInput[ inPtr++] = inChr;
          msgSize--;
          if( 0 == msgSize){
            chrState = RX_CHECKSUM;
          }
          break;
        case RX_CHECKSUM:
          rxInput[ inPtr++] = inChr;
          if( checkSum){ //  Did not come out to zero...
            errorBits |= CHECKSUM_ERROR;
          }
          chrState = RX_TAIL;
          break;
        case RX_TAIL:
          rxInput[ inPtr++] = inChr;
          if( TRAILCHAR != inChr){
            errorBits |= TRAIL_ERROR;
          }
          chrState = RX_COMPLETE;
          break;
        case RX_COMPLETE:
          if( errorBits){
            sendNAKmsg();
          }
          parseInput( (data_packet_head_t *)rxInput);
        default:
          errorBits |= STATE_ERROR;
          break;
      }
      inChr = UART1_UartGetChar();
    }while(1);
  }
  return 0;
}
/**
  * @brief - Routine to parse input buffer
  * @param - p - pointer to input buffer
  * @param - count - number of bytes in buffer
  * @retval - success of operation
  */
int32_t parseInput( data_packet_head_t *packet){
  uint8_t *p = (uint8_t *)&packet->datum1;
  uint8_t count = packet->count;
  uint32_t retStat;
  
  switch( *p){
    case CLEARFLASH:
      clearLogFile();
      sendACKmsg();
      retStat = 0;
      break;
    case READFLASH:
      dumpLogFile();
      sendACKmsg();
      retStat = 0;
      break;
    case SYSVARS:
      /**
        * message (after count) looks like this:
        * 'S' Tset(H) Tset (L) tCool(H) tCool(L) tRefract(H) tRefract(L)
        * 
        */
      p++;  //  Get past the 'S'
      sys_var_t *sp = (sys_var_t *)p;
      tSetpoint = (float) sp->tSet;
      coolDuration = sp->tCool;
      refractDuration = sp->tRefr;
      sendACKmsg();
      retStat = 0;
      break;
    case SETTIME:
      /**
        * This message looks like
        * 'S' YR MO DA HR MN SC in 8 bit fields
        */
      p++;
      time_var_t *st = (time_var_t *)p;
      rtc_time_t t;
      t.year = st->year;
      t.month = st->month;
      t.date = st->day;
      t.hrs = st->hour;
      t.mins = st->minute;
      t.secs = st->sec;
      t.hund = 0;
      if( (retStat = rtc_write_clock( RTC_ADDR, &t))){
        sendNAKmsg();
      }
      else{
        sendACKmsg();
      }
      break;
    case READSYSVARS:
      break;
    case READSYSTIME:
      break;
    case ACKPACKET:
      break;
    case NAKPACKET:
      break;
    default:
      break;
  }
  return retStat;
}
/* [] END OF FILE */
