/**
  ******************************************************************************
  * @file    	comms.h
  * @brief   	Control communications between device and PC
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#ifndef __COMMS_H_
#define __COMMS_H_
#include  "project.h"
  
/**
  * @note   This module implements the communications protocol between the device
  *     and a PC or other device used to configure the device and gather data logs.
  *
  * According to the specification for this protocol, there are:
  *   a header byte 'C' (0x43)
  *   a sequence number
  *   a count
  *   a message of some count bytes
  *   a checksum 
  *   a trailer byte 'D' (0x44)
  *   I was not clever enough to dictate the size of the fields.
  * Traditionally, the data looks like this:
  
struct out_data_packet{               // transmit data structure
   unsigned int8   header;
   unsigned int8   version;
   unsigned int8   control1;
   unsigned int8   control2;
   signed int32   EEGWav;
   unsigned int16   PacketCnt;      // A counter for the packets sent                        
   unsigned int8   crc;
   unsigned int8   trailer;
   } data_output;

struct in_data_packet{               // receive data structure
   unsigned int8   header;
   unsigned int8   control1;
   unsigned int8   crc;
   unsigned int8   trailer;
   } data_input;
  *
  * which is a format Eric came up with. I think it looks OK for sizing, so I'll
  * incorporate this.
  */
typedef struct{
 uint8_t header;		//	‘C’
 uint8_t version;	//	‘0’
 uint8_t sequence;	//	0..255 – just keeps rolling
 uint8_t count;		//	< 255
 uint8_t datum1;
}data_packet_head_t;
// uint8_t data[];		//	fewer than 255 bytes - Really an array, but C is dumb in this regard...
typedef struct{
 uint8_t checksum;	//	XOR of header through last byte of data
 uint8_t trailer;		//	‘D’
}data_packet_tail_t;

typedef struct{
  uint16_t  tSet;
  uint16_t  tCool;
  uint16_t  tRefr;
}sys_var_t;
typedef struct{
  uint8_t year;
  uint8_t month;
  uint8_t day;
  uint8_t hour;
  uint8_t minute;
  uint8_t sec;
}time_var_t;
/** 
  * A relatively simple state machine can take care of the message
  * gathering.
  */
typedef enum{
  RX_START = 0,
  RX_VERSION,
  RX_SEQUENCE,
  RX_COUNT,
  RX_DATA,
  RX_CHECKSUM,
  RX_TAIL,
  RX_COMPLETE
}rx_states_t;

#define HEADERCHAR    'C'
#define TRAILCHAR     'D'
#define CLEARFLASH    'Z'   //  ACK response
#define READFLASH     'R'   //  Dump of all flash response
#define SYSVARS       'S'   //  ACK response
#define SETTIME       'T'   //  ACK response
#define READSYSVARS   'V'   //  Sysvars response
#define READSYSTIME   '?'   //  Systime response
#define ACKPACKET     0x6
#define NAKPACKET     0x15
#define VERSIONCHAR   0x10  //  Version 1 . 0

/**
  * Error bits
  */
#define VERSION_ERROR  1
#define CHECKSUM_ERROR 2
#define SEQUENCE_ERROR 4
#define TRAIL_ERROR    8
#define STATE_ERROR    16

uint32_t  dumpLogFile( void);
uint32_t  clearLogFile( void);
uint32_t  configSystemVariables( void);
uint32_t  configSystemTime( void);
uint32_t  readSystemVariables( void);
uint32_t  readSystemTime( void);
int32_t interpret( uint8_t in);
int32_t parseInput( data_packet_head_t *packet);
int32_t sendNAKmsg( void);
int32_t sendACKmsg( void);


#endif  //__COMMS_H_
/* [] END OF FILE */
