/**
  ******************************************************************************
  * @file    	fram.h
  * @brief   	Control FRAM chip via I2C
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#include  "fram.h"
#include  "logging.h"
#include  "timebase.h"
/**
  * @note The FRAM consists of 1024 blocks of 64 pages of 2048 bytes.
  */
uint32_t FRAMAddress;
uint32_t FRAMPage;
uint32_t FRAMBlock;
uint32_t  blockAddress;
uint32_t  pageAddress;

uint8_t fWrBuffer[ FRAM_PAGE_SIZE];
uint8_t fRdBuffer[ FRAM_PAGE_SIZE];


/**
  * @note This module is a bit odd. The FRAM chip is on the other side of a
  *       I2C-to-SPI bridge. The bridge requires control data to convey 
  *       the operation to perform.
  *       A write operation consists of 
  *       a Page Write command with data followed by a Program Execute command.
  *       A read operation consist of a Page Read Data command 
  *       followed by a Read Data command.
  *       Pay heed that in order to get data out of a SPI device,
  *       one must push dummy data to the SPI device...
  */
static uint8_t readBuffer[RW_BUFFER_SIZE];   //  Intermediate read buffer (I2C reads into here, then we copy it out)
static uint8_t writeBuffer[RW_BUFFER_SIZE];  //  Intermediate write buffer
static uint8_t controlBuffer[10]; //  command buffer
static uint8_t replyBuffer[10];
/**
  * @brief  Calculate the address offset
  * @param  page to convert
  * @retval address offset
  * @note   FRAM is composed of 2048-byte pages.
  */
uint16_t  block_address( uint32_t page){
  uint32_t address = page * FRAM_PAGE_SIZE;
  return ( address & 0x7FF);
}
/**
  * @brief    Initialize the FRAM interface
  * @param    none  
  * @retval   status of the bridge configuration
  */
uint32_t init_fram( void){
  uint32_t  status;    
  FRAM_Ena_Write( 1);   //  Controls power to FRAM - just turn it on.
  status = bridge_config_spi( FRAM_ADDR, 0x00);   //  Default configuration
  return status;
}
/**
  * @brief  -  fill a buffer with a pattern of 0001, 0002, 0003, etc
  * @param    buffer - buffer pointer to fill
  * @param    count  - number of bytes to fill
  * @retval   none
  */
void set_buffer_pattern( uint8_t *buffer, uint32_t count){
  uint16_t *t = (uint16_t *)buffer;
  uint32_t wCount = count/2;
  for( uint32_t i = 0; i < wCount; i++){
    t[ i] = i;
  }
}

/**
  * @brief  -  fill a buffer with a value
  * @param    buffer - buffer pointer to fill
  * @param    count  - number of bytes to fill
  * @retval   none
  */
void set_buffer_to( uint8_t *buffer, uint32_t count, uint8_t value){
  uint16_t *t = (uint16_t *)buffer;
  uint16_t wVal = (value << 8) + value;
  uint32_t wCount = count/2;
  for( uint32_t i = 0; i < wCount; i++){
    t[ i] = wVal;
  }
}

/**
  * @brief  Read data from FRAM to FRAM buffer, then transfer back, 128 bytes at a throw
  * @param  slaveAddress - address of bridge
  * @param  page - index of page (0..65k) to read
  * @param  rdAddress - address in FRAM to read
  * @param  count - number of bytes to transfer
  * @retval status of operation
  * @note   Calls fram_page_data_read() and fram_read_buffer()
  */
uint32_t fram_read( uint8_t slaveAddress, uint8_t *rdData, uint32_t page, uint32_t count){
  uint32_t status;
  uint16_t blockAddress;
  uint8_t tryCount = 10;
  uint8_t *pData = rdData;
  do{
    status = fram_page_data_read( slaveAddress, page);
    if( status){
      I2CM_I2CMasterSendStop( I2C_DELAY);
      --tryCount;
    }
    if( 0 == tryCount){
        break;
    }
  }while( status);
  
  /** 
    * @note - Calculate the correct page count. I anticipate always getting
    * pages of 2048 bytes, but don't assume anything too broadly.
    * If the count is not an even multiple of the buffer size (128 bytes)
    * then bump the page count
    */
  uint32_t page_count = count / FRAM_BUFFER_SIZE;  
  if( count % FRAM_BUFFER_SIZE){
    page_count++;
  }
  if( page_count > FRAM_PAGE_SIZE/FRAM_BUFFER_SIZE){
    page_count = FRAM_PAGE_SIZE / FRAM_BUFFER_SIZE;
  }
  
  if( 0 == status){
    for( uint32_t i = 0; i < page_count; i++){
      tryCount = 10;
      do{
        blockAddress = FRAM_BUFFER_SIZE * i;  //  buffers of 128 bytes
        status = fram_read_buffer( slaveAddress, pData, blockAddress, FRAM_BUFFER_SIZE);
        pData += FRAM_BUFFER_SIZE;
        //status = fram_read_buffer( slaveAddress, &rdData[(int) block_address], blockAddress, FRAM_BUFFER_SIZE);
        //status = fram_read_buffer( slaveAddress, rdData, blockAddress, FRAM_BUFFER_SIZE); //  TODO: This is wrong
        
        if( status){
          I2CM_I2CMasterSendStop( I2C_DELAY);
          --tryCount;
        }
        if( 0 == tryCount){
          break;
        }
      }while( status);
    }
  }
  return status;
}
/**
  * @brief  Read a page of data from FRAM into FRAM buffer
  * @param  slaveAddress - address of bridge
  * @param  pageAddress - address in FRAM (ordinarily should be zero)
  * @retval status of operation
  */
uint32_t fram_page_data_read( uint8_t slaveAddress, uint32_t page){
  uint32_t status;
  
  writeBuffer[0] = FRAM_PAGE_READ;
  writeBuffer[1] = DUMMY_BYTE;  //  dummy byte
  writeBuffer[2] = page >> 8;
  writeBuffer[3] = page & 0xFF;
  status = i2c_query_fram( slaveAddress, writeBuffer, 4);
  
  return status;
}
/**
  * @brief  Read FRAM buffer
  * @param  slaveAddress - address of bridge
  * @param  rdData - pointer to buffer
  * @param  offset  - offset in buffer to read
  * @param  count - number of bytes to read
  * @retval status of operation
  * @note   *** Assumes there is meaningful data in the buffer, i.e. after a fram_page_data_read()
  *         or fram_write_buffer()
  */
uint32_t fram_read_buffer( uint8_t slaveAddress, uint8_t *rdData, uint32_t blockAddress, uint32_t count){
  uint32_t status;
  uint32_t  opcode = SPI_READ_WRITE;
  CYASSERT( count < 190);   //  Don't want any overflow of the Bridge's 200 byte buffer
  // TODO: Rewrite this with a buffer write
  I2CM_I2CMasterClearStatus();

  status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
  if( status){
    I2CM_I2CMasterSendStop( I2C_DELAY);
    return 0xE1; //status;
  }
  else{
    status = I2CM_I2CMasterWriteByte( opcode, I2C_DELAY);
    if( status){
      I2CM_I2CMasterSendStop( I2C_DELAY);
      return 0xE2; //status;
    }
    else{
      status = I2CM_I2CMasterWriteByte(FRAM_RD_BUFFER, I2C_DELAY);
      if(status){
        I2CM_I2CMasterSendStop( I2C_DELAY);
        return 0xE3; //status;
      }
      else{
        status = I2CM_I2CMasterWriteByte(blockAddress >> 8, I2C_DELAY);
        if( status){
          I2CM_I2CMasterSendStop( I2C_DELAY);
          return 0xE4; //status;
        }
        else{
          status = I2CM_I2CMasterWriteByte( blockAddress & 0xFF, I2C_DELAY);
          if( status){
            I2CM_I2CMasterSendStop( I2C_DELAY);
            return 0xE5; //status;
          }
          else{
            status = I2CM_I2CMasterWriteByte( DUMMY_BYTE, I2C_DELAY);
            if( status){
              I2CM_I2CMasterSendStop( I2C_DELAY);
              return 0xE6; //status;
            }
            else{
              for( uint32_t i = 0; i < count; i++){
                status = I2CM_I2CMasterWriteByte( DUMMY_BYTE, I2C_DELAY);
                if( status){
                  I2CM_I2CMasterSendStop( I2C_DELAY);
                  return 0xE7; //status;
                }
              }
            }

            status = I2CM_I2CMasterSendStop( I2C_DELAY);
            if( status){
              return 0xE8; //status;
            }
            CyDelay( 10);
           
            status = I2CM_I2CMasterSendStart( slaveAddress, I2CM_I2C_READ_XFER_MODE, I2C_DELAY);

            if( status){
              I2CM_I2CMasterSendStop( I2C_DELAY);
              return 0xE9; //status;
            }
            else{
              /** 
                * @note And all this time I was not skipping the 4-byte
                *       preamble of the command, address, address, dummy \
                */
              for( uint32_t i = 0; i < 4; i++){
                if( I2CM_I2CMasterReadByte(I2CM_I2C_ACK_DATA, &rdData[i], I2C_DELAY)){
                  status = TRANSFER_ERROR;
                  I2CM_I2CMasterSendStop(I2C_DELAY);
                  return 0xEA; //status;
                }
              }
              /** 
                * @note NOW we can read the data back
                */
              for( uint32_t i = 0; i < count-1; i++){
                if( I2CM_I2CMasterReadByte(I2CM_I2C_ACK_DATA, &rdData[i], I2C_DELAY)){
                  status = TRANSFER_ERROR;
                  I2CM_I2CMasterSendStop(I2C_DELAY);
                  return 0xEA; //status;
                }
              }
              if( I2CM_I2CMasterReadByte( I2CM_I2C_NAK_DATA, &rdData[count-1], I2C_DELAY)){
                status = TRANSFER_ERROR;
                I2CM_I2CMasterSendStop(I2C_DELAY);
                return 0xEB; //status;
              }
            }
          }
        }
      }
    }
  }
  I2CM_I2CMasterSendStop( I2C_DELAY);  
  return status;
}
/**
  * @brief  Copy data to FRAM buffer and program into FRAM
  * @param  slaveAddress - address of SPI bridge
  * @param  pageAddress - page to erase
  * @retval status of operation
  * @note   calls fram_write_buffer() and fram_program_execute()
  */
uint32_t fram_write(uint8_t slaveAddress, uint8_t *wrData, uint32_t pageAddress, uint32_t count){
  uint32_t status;
  uint32_t  xfer_count;
  uint64_t  timeOut;
  uint8_t opStatus;

  
  /** 
    * @note - Calculate the correct page count. I anticipate always getting
    * pages of 2048 bytes, but don't assume anything too broadly.
    * If the count is not an even multiple of the buffer size (128 bytes)
    * then bump the page count
    */
  xfer_count = count / FRAM_BUFFER_SIZE;  
  if( count % FRAM_BUFFER_SIZE){
    xfer_count++;
  }
  if( xfer_count > FRAM_PAGE_SIZE/FRAM_BUFFER_SIZE){
    xfer_count = FRAM_PAGE_SIZE / FRAM_BUFFER_SIZE;
  }
  /**
    * @note - first thing to do is disable write protection
    */
  status = fram_enable_write_protection( slaveAddress, WP_DISABLED);
  /** 
    * @note - and enable writing - not enough to just
    * remove write protection
    */
  if( status){
    print_error( "Fail to disable write protection on FRAM");
    fram_enable_write_protection( slaveAddress, WP_DISABLED);
    return status;
  }
  else{
    fram_enable_write( slaveAddress, WR_ENABLED);  
    fram_get_status( slaveAddress, STAT_REG_C, &opStatus);
    if( 0 == status){
      if( opStatus & 2){
        print_error("Found WEL in fram_write()");
      }
      else{
        print_error("Failed to find WEL in fram_write()");
      }
    }
  }

  /** 
    * @note - now that the write protect is off, we can send data
    */
  for( uint32_t xfer = 0; xfer < xfer_count; xfer++){
    blockAddress = xfer * FRAM_BUFFER_SIZE;
    status = fram_write_buffer( FRAM_ADDR, &wrData[ blockAddress], blockAddress, FRAM_BUFFER_SIZE);

  }
  CyDelay( 10); //  Breakpoint anchor?

  if( 0 == status){
    status = fram_program_execute( slaveAddress, pageAddress);
    timeOut = ERASE_TIMEOUT + now(); //  Op times out in 10mS
    do{
      fram_get_status( slaveAddress, STAT_REG_C, &opStatus);
    }while( now() < timeOut);
  }
  fram_enable_write( slaveAddress, WR_DISABLED);
  fram_enable_write_protection( slaveAddress, WP_ENABLED);

  return status;
}
/**
  * @brief  Write data to FRAM buffer
  * @param  slaveAddress - address of SPI bridge
  * @param  wrData - pointer to incoming data
  * @param  wrAddress - offset in buffer
  * @param  count  - number of bytes to transfer
  * @retval status of operation
  * @note   It appears to be easier to form a complete buffer to send
  *         with opcode, count, and data, rather than piece it all
  *         together calling sendstart(), sendbyte() and sendstop()
  * @note   Prior to sending data, write protection must be turned off,
  *         and write enable must be turned on.
  */
uint8_t wrBuffer[ 200];
uint32_t fram_write_buffer( uint8_t slaveAddress, uint8_t *wrData, uint32_t wrAddress, uint32_t count){
  uint32_t status;
  uint32_t  opcode = SPI_READ_WRITE;
  uint32_t  _count = 4 + count;
  I2CM_I2CMasterClearStatus();

  wrBuffer[ 0] = opcode;
  wrBuffer[ 1] = FRAM_LOAD_RAND;
  wrBuffer[ 2] = wrAddress >> 8;
  wrBuffer[ 3] = wrAddress & 0xFF;
  for( uint32_t i = 4, j = 0; i < _count; i++){
    wrBuffer[ i] = wrData[ j++];
  }
  status = I2CM_I2CMasterWriteBuf( slaveAddress, wrBuffer, _count, I2CM_I2C_MODE_COMPLETE_XFER);
  CyDelay( 20); //  This seems to be necessary ( and 10mS does not cut it!) TODO:
  return status;
}
/**
  * @brief  Execute the programming instruction
  * @param  slaveAddress - address of SPI bridge
  * @param  page - page to erase
  * @retval status of operation
  */
uint32_t fram_program_execute( uint8_t slaveAddress, uint32_t page){
  uint32_t status;
  controlBuffer[0] = FRAM_PROG_EXEC;
  controlBuffer[1] = DUMMY_BYTE;
  controlBuffer[2] = page >> 8;
  controlBuffer[3] = page & 0xFF;
  status = i2c_query_fram( slaveAddress, controlBuffer, 4);
  CyDelay( 1);
  return status;
}
/**
  * @brief  Reset the FRAM
  * @param  slaveAddress - address of SPI bridge
  * @retval status of operation
  */

uint32_t  fram_reset( uint8_t slaveAddress){
  uint32_t status;
  controlBuffer[0] = FRAM_RESET;
  status = i2c_query_fram(slaveAddress, controlBuffer, 1);
  CyDelay( 2);  //  Just for fun  TODO:
  return status;
}
/**
  * @brief  Enable programming of data
  * @param  slaveAddress - address of SPI bridge
  * @param  enable - 1/0 to enable/disable write
  * @retval status of operation
  */

uint32_t fram_enable_write(uint8_t slaveAddress, uint8_t enable){
  uint32_t status;
  
  controlBuffer[0] = enable ? FRAM_WR_ENABLE : FRAM_WR_DISABLE;
  status = i2c_query_fram( slaveAddress, controlBuffer, 1);
  CyDelay( 1);

  return status;
}
/**
  * @brief  Manage the write-protection status of the chip
  * @param  slaveAddress - address of SPI bridge
  * @param  enable - enable or disable
  * @retval status of operation
  * @note   If, when clearing write protection, if the operation 
  *         fails, attempt to restore write protection, regardless
  *         the intended operation.
  */
uint32_t fram_enable_write_protection( uint8_t slaveAddress,  uint8_t enable){
  uint32_t  status;
  uint8_t stat_reg;
  
  
  fram_get_status( slaveAddress, 0xA0, &stat_reg);

  /** Clearing bits BP3, BP2, BP1, BP0, TB in Ax register 
    * clears write protection for the entire device.
    * Setting them all protects the entire device.
    */
  if( WP_ENABLED == enable){
    stat_reg |= 0x7C;   //  0111 1100 - Set the protect bits
//    stat_reg = 0xFF;      //  Register A0 reads back 00, but we don't
                          //  want to leave bits SRP0, SRP1, and WP-E at zero, do we?
  }
  else{
    stat_reg &= 0x83; //1000 0011 
  }
  status = fram_set_status( slaveAddress, 0xA0, stat_reg);
  if( status){
    /**
      * @note - on failure, attempt to restore value
      */
    stat_reg |= 0x7C; //  01111100 - protect 
    //stat_reg = 0xFF;
    fram_set_status( slaveAddress, 0xA0, stat_reg);
  }
  CyDelay( 1);
  return status;
}
/**
  * @brief  get JEDEC ID of chip
  * @param  slaveAddress - address of SPI bridge
  * @param  buffer  - pointer to buffer for data
  * @retval status of operation
  */

uint32_t fram_get_JEDEC( uint8_t slaveAddress, uint8_t *buffer){
  uint32_t status;
  uint8_t chip_ID[3] = { 0xEF, 0xAA, 0x21};
  controlBuffer[0] = FRAM_RD_JEDEC;
  controlBuffer[1] = 0xFF;  //  dummy byte
  controlBuffer[2] = 0xFF;  //  dummy byte
  controlBuffer[3] = 0xFF;  //  dummy byte
  controlBuffer[4] = 0xFF;  //  dummy byte
  controlBuffer[5] = 0xFF;  //  dummy byte
  status = i2c_query_fram(slaveAddress, controlBuffer, 5);
  CyDelay( 1);
  /**
    * @note Bytes are offset by two leading bytes. Fixup for now
    * TODO:
    */
  if( 0 == status){
    status = i2c_read_fram( slaveAddress, replyBuffer, 5); //  same number of bytes as sent!
    /**
      * @note   Read back the ID OK. Simultaneously copy to
      *         return buffer and vet against known
      *         chip ID: EF AA 21
      */
    if( 0 == status){
      for( uint32_t i = 0, j = 2; i < 3; i++, j++){
        buffer[i] = replyBuffer[j];
        if( buffer[ i] != chip_ID[ i]){
          status |= 1;
        }
      }
    }
  }
  return status;
}
/**
  * @brief  get status of FRAM
  * @param  slaveAddress - address of SPI bridge
  * @param  sr - pointer to buffer passed in
  * @retval status of operation
  */
uint32_t fram_get_status( uint8_t slaveAddress, uint8_t regAddress, uint8_t *sr){
  uint32_t status;
  controlBuffer[0] = FRAM_RD_STATUS;
  controlBuffer[1] = regAddress;
  controlBuffer[2] = 0xFF;  //  dummy byte
  controlBuffer[3] = 0xFF;  //  dummy byte
  controlBuffer[4] = 0xFF;  //  dummy byte
  controlBuffer[5] = 0xFF;  //  dummy byte
  /**
    * @note Reply is coming in padded with two leading bytes.
    *     I suspect this is either a bug on my part or a
    *     flake in the bridge chip.
    *     TODO:
    */
  status = i2c_query_fram( slaveAddress, controlBuffer, 4);
  if( 0 == status){
    status = i2c_read_fram( slaveAddress, replyBuffer, 3);
    if( 0 == status){
      *sr = replyBuffer[2];
    }
  }
  
  return status;
}
/**
  * @brief  Write to the FRAM status register
  * @param  slaveAddress - address of SPI bridge
  * @param  regAddress - which register (Ax, Bx, or Cx)
  * @param  value - value to set
  * @retval status of operation
  */
uint32_t fram_set_status( uint8_t slaveAddress, uint8_t regAddress, uint8_t value){
  uint32_t status;
  controlBuffer[0] = FRAM_WR_STATUS;
  controlBuffer[1] = regAddress;
  controlBuffer[2] = value;
  status = i2c_query_fram( slaveAddress, controlBuffer, 3);
  return status;
}

/**
  * @brief  Erase the 128KB block
  * @param  slaveAddress - address of SPI bridge
  * @param  pageAddress - page to erase
  * @retval status of operation
  */
uint32_t  fram_erase_block( uint8_t slaveAddress, uint32_t block){
  uint32_t status;
  uint8_t tryCount;
  uint8_t opStatus;
  uint64_t timeOut;

/**
  * What does C0 look like before killing write protection?
  */
  fram_get_status( slaveAddress, STAT_REG_C, &opStatus);
  CyDelay( 1);
  
  status = fram_enable_write_protection( slaveAddress, WP_DISABLED);
  if( status){
    fram_enable_write_protection( slaveAddress, WP_ENABLED);
    print_error("Failed to disable write protection in erase_block()");
    return status;
  }
/**
  * What does C0 look like before enabling write?
  */
  fram_get_status( slaveAddress, STAT_REG_C, &opStatus);
  CyDelay( 1);

  status = fram_enable_write( slaveAddress, WR_ENABLED);
  status = fram_get_status( slaveAddress, STAT_REG_C, &opStatus);
  if( 0 == status){
    if( opStatus & 2){
      print_error("Found WEL bit in fram_erase_block()");
    }
  }
  if( 0 == status){
    tryCount = 10;
    controlBuffer[0] = FRAM_BLOCK_ERASE;
    controlBuffer[1] = DUMMY_BYTE;
    controlBuffer[2] = block >> 8;
    controlBuffer[3] = block & 0xFF;
    do{
      status = i2c_query_fram( slaveAddress, controlBuffer, 4); 
      if( status){
        I2CM_I2CMasterSendStop( I2C_DELAY);
        --tryCount;
      }
      if( 0 == tryCount){
        print_error( "Failed to erase in erase_block()");
        fram_enable_write( slaveAddress, WR_DISABLED);
        fram_enable_write_protection( slaveAddress, WP_ENABLED);
        return status;
        break;
      }
    }while( status);
  }
  else{
    print_error("Failed to enable write in erase_block()");
    fram_enable_write( slaveAddress, WR_DISABLED);
    fram_enable_write_protection( slaveAddress, WP_ENABLED);
    return status;
  }
  /**
    * @note wait 2..10mS for the operation to complete before
    *     clearing the write enable & setting write protection
    */
  CyDelay( 2);
  
  timeOut = ERASE_TIMEOUT + now(); //  Op times out in 10mS
  do{
    fram_get_status( slaveAddress, STAT_REG_C, &opStatus);
  }while( now() < timeOut);
  
  CyDelay( 1);
  fram_enable_write( slaveAddress, WR_DISABLED);
  
  CyDelay( 1);
  fram_enable_write_protection( slaveAddress, WP_ENABLED);
  
  CyDelay( 1);
/**
  * What does C0 look like after disabling write?
  */
  fram_get_status( slaveAddress, STAT_REG_C, &opStatus);
  CyDelay( 1);
  
  return status;
};
/** 
  * @brief  Clear interrupt on bridge
  * @param  slaveAddress - address of bridge
  * @retval status of operation
  */
uint32_t bridge_clear_interrupt( uint8_t slaveAddress){
  uint32_t status;
  controlBuffer[0] = SPI_CLR_INTERRUPT;
  status = i2c_write_bridge( slaveAddress, controlBuffer, 1);
  
  return status;
}
/**
  * @brief  Bring bridge to low power state
  * @param  none
  * @retval status of operation
  */
uint32_t bridge_idle( uint8_t slaveAddress){
  uint32_t status;
  controlBuffer[0] = SPI_IDLE_MODE;
  status = i2c_write_bridge( slaveAddress, controlBuffer, 1);
  
  return status;
}
/**
  * @brief  Enable GPIO for the bridge
  * @param  slaveAddress - address of the bridge
  * @param  mask - bitmask of which bits (3, 2, 1, 0) is active
  * @retval  status
  * @note   This was included to check the interface to the FRAM
  */
uint32_t bridge_enable_gpio( uint8_t slaveAddress, uint8_t mask){
  CYASSERT( 0 == (mask & 0xF1));  //  We want to ensure that SS0 remains as chip select
  uint32_t status;
  controlBuffer[0] = SPI_GPIO_ENABLE;
  controlBuffer[1] = mask;
  status = i2c_write_bridge( slaveAddress, controlBuffer, 2);
  
  return status;
}

/**
  * @brief  Configure GPIO for the bridge
  * @param  slaveAddress - address of the bridge
  * @param  mask - bitmask of which bits (3, 2, 1, 0) is active
  *                 in which mode
  * @retval  status
  * @note   This was included to check the interface to the FRAM
  */
uint32_t bridge_configure_gpio( uint8_t slaveAddress, uint8_t mask){
  uint32_t status;
  controlBuffer[0] = SPI_GPIO_CONFIG;
  controlBuffer[1] = mask;
  status = i2c_write_bridge( slaveAddress, controlBuffer, 2);
  
  return status;
}

/**
  * @brief  Read GPIO from the bridge
  * @param  slaveAddress - address of the bridge
  * @param  buffer - pointer to storage area
  * @retval  status
  * @note   This was included to check the interface to the FRAM
  */
uint32_t bridge_read_gpio( uint8_t slaveAddress, uint8_t *buffer){
  uint32_t status;
  controlBuffer[0] = SPI_GPIO_READ;
  controlBuffer[1] = DUMMY_BYTE;
  controlBuffer[2] = DUMMY_BYTE;
  status = i2c_query_bridge( slaveAddress, controlBuffer, 2);
  if( 0 == status){
    controlBuffer[0] = DUMMY_BYTE;
    controlBuffer[1] = DUMMY_BYTE;
    controlBuffer[2] = DUMMY_BYTE;
    status = i2c_read_bridge( slaveAddress, readBuffer, 1);
    if( 0 == status){
      memcpy( buffer, readBuffer, 2);
    }
  }
  return status;
}

/**
  * @brief  Write GPIO to the bridge
  * @param  slaveAddress - address of the bridge
  * @param  value - value to write
  * @retval  status
  * @note   This was included to check the interface to the FRAM
  */
uint32_t bridge_write_gpio( uint8_t slaveAddress, uint8_t value){
  uint32_t status;
  CYASSERT( 0 == (value & 0xF1)); //  Only bits 3,2,1,0 available - we use 0
  controlBuffer[0] = SPI_GPIO_WRITE;
  controlBuffer[1] = value;
  status = i2c_write_bridge( slaveAddress, controlBuffer, 2);
  
  return status;
}
/**
  * @brief  Set up bridge's SPI configuration
  * @param  slaveAddress - I2C address of the SPI bridge
  * @param  mask - value to program into the register
  * @retval result of operation
  */
uint32_t bridge_config_spi( uint8_t slaveAddress, uint8_t mask){
  uint32_t status;
  controlBuffer[0] = SPI_CONF_INTFC;
  controlBuffer[1] = mask;
  status = i2c_write_bridge( slaveAddress, controlBuffer, 2);
  
  return status;
}
/**
  * @brief  Scan the FRAM for marked bad blocks
  * @param  slaveAddress - I2C address of the bridge
  * @param  badBlocks - pointer to table storage
  * @param  maxCount - number of elements we can save.
  * @retval status of the operation
  * @note   blocks are 128KB long - at increment of 0x40 pages ( 131,072 bytes)
  * 
  */
uint32_t fram_bad_blocks( uint8_t slaveAddress, uint32_t *badBlocks, uint32_t maxCount){
  uint32_t bbIndex = 0;
  uint32_t block;
  uint32_t status;
  
  for( block = 0; block < FRAM_MAX_BLOCK; block++){
    status = fram_read( slaveAddress, readBuffer, block, FRAM_PAGE_SIZE);
    if( 0 == status){
      if( readBuffer[ 0 ] != 0xFF){   //  bad block marker
        badBlocks[ bbIndex] = block;
        if( ++bbIndex > maxCount){
          return 1; //  This is not good
        }
      }
    }
  }
  return 0; //  We have escaped at least relatively unscathed
}
/**
  * @brief  Routine to erase the entire fram
  * @param  slaveAddress = I2C address of bridge
  * @retval success of operation
  * @note   The entire FRAM is erased. Any error reported, but 
  *         not specifically...
  */
uint32_t fram_erase_fram( uint8_t slaveAddress){
  uint32_t status = 0;
  for( uint32_t block = 0; block < FRAM_MAX_BLOCK; block++){
    status |= fram_erase_block( slaveAddress, block);
  }
  return 0;
}
/**
  * @brief  Exhaustive test of the FRAM
  * @param  none
  * @retval Status of the test
  */
int8_t  tryCount;
int32_t badFlag;
int64_t errorCount = 0;
uint16_t *t;

uint32_t  exhaustive_fram_test( void){
  uint32_t  status;
  uint32_t  loggedErrors = 0;
/**
  * Reset the FRAM
  */
  fram_reset( FRAM_ADDR);
/**
  * Read the FRAM before erase
  */
  for( uint32_t page = 0; page < PAGE_COUNT; page++){
    badFlag = 0;
    tryCount = 10;
    do{
      set_buffer_to( fRdBuffer, 2048, 0);
      status = fram_read( FRAM_ADDR, fRdBuffer, page, FRAM_PAGE_SIZE);
      if( I2CM_I2C_MSTR_NOT_READY == status){
        I2CM_I2CMasterSendStop( I2C_DELAY);
      }
      if( 0 == --tryCount){
        print_error( "Failed to fram_read() in main()\n\r");
        break;
      }
    }while( status);
    if( status){
      loggedErrors++;
      }
    /**
      * Fix up a compare buffer
      */
    uint16_t *t = (uint16_t *)fWrBuffer;
    for( uint32_t i = 0; i < 1024; i++){
      t[ i] = i;
    }
    t[ 0] = 0x3300 + page;
    
    if( 0 == status){
      for( uint32_t i = 0; i < FRAM_BUFFER_SIZE; i++){
        if( fRdBuffer[i] != fWrBuffer[i]){
          badFlag = i;
        }
      }
      if( badFlag){
        print_error( "Bad page read in main() to start\n\r");
        loggedErrors++;
      }
    }
  }

/**
  * Erase the FRAM
  */
  tryCount = 10;
  do{
    status = fram_erase_fram( FRAM_ADDR);
    CyDelay( 10);
    if( 0 == --tryCount)
      break;
  }while (status );
  if( status){
    CyDelay( 10);
    loggedErrors++;
  }
/**
  * Reset the FRAM
  */
  fram_reset( FRAM_ADDR);
/**
  * Read the FRAM after Erase
  */
  for( uint32_t page = 0; page < PAGE_COUNT; page++){
    do{
      set_buffer_to( fRdBuffer, 2048, 0);
      status = fram_read( FRAM_ADDR, fRdBuffer, page, FRAM_PAGE_SIZE);
      if( I2CM_I2C_MSTR_NOT_READY == status){
        I2CM_I2CMasterSendStop( I2C_DELAY);
      }
      if( 0 == --tryCount){
        print_error( "Failed to fram_read() in main()");
        break;
      }
    }while( status);
    if( status){
      loggedErrors++;
      }
    /**
      * Compare the erase buffer
      */
    t = (uint16_t *)fRdBuffer;
    badFlag = 0;
    for( int i = 0; i < 1024; i++){
      if( t[ i] != 0xFFFF){
        badFlag |= 1;
      }
    }
    if( badFlag){
      print_error( "Bad page erase in main()\n\r");
      loggedErrors++;
    }
  }
/**
  * Reset the FRAM
  */
  fram_reset( FRAM_ADDR);

/**
  * Program the FRAM - full pages
  */
  set_buffer_pattern( fWrBuffer, FRAM_PAGE_SIZE);
  for( uint32_t page = 0; page < PAGE_COUNT; page++){
    fWrBuffer[ 0] = 0x33;
    fWrBuffer[ 1] = 0x33;
    fWrBuffer[ 2] = page >> 8;
    fWrBuffer[ 3] = page & 0xFF;
    status = fram_write( FRAM_ADDR, fWrBuffer, page, FRAM_PAGE_SIZE);
    if( status){
      loggedErrors++;
      }  
  }
/**
  * Reset the FRAM
  */
  fram_reset( FRAM_ADDR);

/**
  * Read the FRAM
  */
  for( uint32_t page = 0; page < PAGE_COUNT; page++){
    set_buffer_to( fRdBuffer, FRAM_PAGE_SIZE, 0);
    tryCount = 10;
    do{
      status = fram_read( FRAM_ADDR, fRdBuffer, page, FRAM_PAGE_SIZE);
      if( I2CM_I2C_MSTR_NOT_READY == status){
        I2CM_I2CMasterSendStop( I2C_DELAY);
      }
      if( 0 == --tryCount){
        print_error( "Failed to fram_read() in main()\n\r");
        break;
      }
    }while( status);
    if( status){
      loggedErrors++;
    }
    /**
      * Fix up a compare buffer
      */
    badFlag = 0;
    set_buffer_pattern( fWrBuffer, FRAM_PAGE_SIZE);
    fWrBuffer[ 0] = 0x33;
    fWrBuffer[ 1] = 0x33;
    fWrBuffer[ 2] = page >> 8;
    fWrBuffer[ 3] = page & 0xFF;
    
    if( 0 == status){
      for( uint32_t i = 0; i < FRAM_BUFFER_SIZE; i++){
        if( fRdBuffer[i] != fWrBuffer[i]){
          badFlag++;
          break;
        }
      }
    }
    if( badFlag){
      loggedErrors++;
    }
  }
/**
  * Reset the FRAM
  */
  fram_reset( FRAM_ADDR);
  return loggedErrors;
}
/* [] END OF FILE */

