/**
  ******************************************************************************
  * @file    	fram.h
  * @brief   	Control FRAM chip via I2C
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#ifndef __FRAM_H
#define __FRAM_H
#include  "project.h"
#include  "I2CM.h"
#include  "i2c.h"
  
#define FRAM_ADDR   0x28
#define RW_BUFFER_SIZE  200
#define FRAM_PAGE_SIZE   2048
#define FRAM_BUFFER_SIZE  128 //  16 buffers per page
                              //  N.B. the buffer in the Bridge is 200 bytes.
#define FRAM_MAX_BLOCK  1024
#define FRAM_MAX_PAGE   64
#define PAGE_COUNT 512
  
#define STAT_REG_A      0xA0
  #define SRP0_MASK 0x80
  #define BP3_MASK  0x40
  #define BP2_MASK  0x20
  #define BP1_MASK  0x10
  #define BP0_MASK  0x08
  #define TB_MASK   0x04
  #define WPE_MASK  0x02
  #define SRP1_MASK 0x01
#define STAT_REG_B      0xB0
  #define OTPL_MASK 0x80
  #define OTPE_MASK 0x40
  #define SR1L_MASK 0x20
  #define ECCE_MASK 0x10
  #define BUF_MASK  0x08
  //Bits 2, 1, 0 are reserved
#define STAT_REG_C      0xC0
  // Bi7 7 is reserved
  #define LUTF_MASK 0x40
  #define ECC1_MASK 0x20
  #define ECC0_MASK 0x10
  #define PFAIL_MASK 0x08
  #define EFAIL_MASK 0x04
  #define WEL_MASK  0x02
  #define BUSY_MASK 0x01
  
#define ERASE_TIMEOUT 10    //  Block erase takes 2..10 mS  
#define PROGRAM_TIMEOUT 2   //  The book says < 700uS. Let's allow 2000?

/**
  * @note Keeping track of data in the FRAM.
  * The FRAM consists of 1024 blocks of 64 pages of 2048 bytes, or more simply,
  * 65536 pages (0..65535) of 2048 bytes.
  */
typedef struct{
  uint32_t  currentPage;
  uint32_t  currentIndex;
}log_values_t;

uint32_t fram_bad_blocks( uint8_t slaveAddress, uint32_t *badBlocks, uint32_t maxCount);
uint32_t fram_read( uint8_t slaveAddress, uint8_t *rdData, uint32_t page, uint32_t count);
uint32_t fram_page_data_read( uint8_t slaveAddress, uint32_t pageAddress);
uint32_t fram_read_buffer( uint8_t slaveAddress, uint8_t *rdData, uint32_t offset, uint32_t count);

uint32_t fram_write(uint8_t slaveAddress, uint8_t *wrData, uint32_t page, uint32_t count);
uint32_t fram_write_buffer( uint8_t slaveAddress, uint8_t *wrData, uint32_t wrAddress, uint32_t count);
uint32_t fram_program_execute( uint8_t slaveAddress, uint32_t pageAddress);

uint32_t fram_reset( uint8_t slaveAddress);
uint32_t fram_enable_write(uint8_t slaveAddress, uint8_t enable);
uint32_t fram_enable_write_protection( uint8_t slaveAddress,  uint8_t enable);
uint32_t fram_get_JEDEC( uint8_t slaveAddress, uint8_t *buffer);
uint32_t fram_get_status( uint8_t slaveAddress, uint8_t regAddress, uint8_t *sr);
uint32_t fram_set_status( uint8_t slaveAddress, uint8_t regAddress, uint8_t value);
uint32_t fram_erase_block( uint8_t slaveAddress, uint32_t page);
uint32_t fram_erase_fram( uint8_t slaveAddress);
uint32_t fram_read_fram( uint8_t slaveAddress);
void set_buffer_to( uint8_t *buffer, uint32_t count, uint8_t value);
void set_buffer_pattern( uint8_t *buffer, uint32_t count);
uint16_t  block_address( uint32_t address);

extern uint32_t blockAddress;
extern uint32_t pageAddress;

uint32_t bridge_enable_gpio( uint8_t slaveAddress, uint8_t mask);
uint32_t bridge_configure_gpio( uint8_t slaveAddress, uint8_t mask);
uint32_t bridge_read_gpio( uint8_t slaveAddress, uint8_t *buffer);
uint32_t bridge_write_gpio( uint8_t slaveAddress, uint8_t value);
uint32_t bridge_config_spi( uint8_t slaveAddress, uint8_t mask);
uint32_t bridge_clear_interrupt( uint8_t slaveAddress);
uint32_t bridge_idle( uint8_t slaveAddress);
uint32_t init_fram( void);
/**
  * @note Message format to the FRAM via the SPI chip needs to follow this format:
  * <SlaveAddress><FunctionID><data0>...<dataN>
  * The buffer is 200 bytes en toto, so data can be up to 198 bytes in theory.
  */
/**
  * @note   The following command codes are from the NXP Semi manual for the
  * @note   SC18IS602B I2C-to-SPI  bridge.
  */
#define SPI_READ_WRITE    0x01
#define SPI_CONF_INTFC    0xF0
#define SPI_CLR_INTERRUPT 0xF1
#define SPI_IDLE_MODE     0xF2
#define SPI_GPIO_WRITE    0xF4
#define SPI_GPIO_READ     0xF5
#define SPI_GPIO_ENABLE   0xF6
#define SPI_GPIO_CONFIG   0xF7

/**
  * @note The following are from the Winbond W25N91GV data sheet
  */
#define FRAM_LOAD_DATA    0x02
#define FRAM_RD_BUFFER    0x03
#define FRAM_WR_DISABLE   0x04
#define FRAM_WR_ENABLE    0x06
#define FRAM_RD_STATUS    0x0F
//#define FRAM_RD_STATUS    0x05
#define FRAM_PROG_EXEC    0x10
#define FRAM_PAGE_READ    0x13
#define FRAM_WR_STATUS    0x1F
//#define FRAM_WR_STATUS    0x01
#define FRAM_LOAD_RAND    0x84
#define FRAM_RD_JEDEC     0x9F
#define FRAM_BAD_BLOCK    0xA1
#define FRAM_READ_BBM     0xA5
#define FRAM_LAST_ECC     0xA9
#define FRAM_BLOCK_ERASE  0xD8
#define FRAM_RESET        0xFF

/**
  * Write ENABLE control
  */
#define WR_ENABLED        1
#define WR_DISABLED       0
/**
  * Write PROTECT control
  * (Don't confuse them - I did)
  */
#define WP_ENABLED        1
#define WP_DISABLED       0

#endif  //  __FRAM_H


/* [] END OF FILE */
