/**
  ******************************************************************************
  * @file    	i2c.c
  * @brief   	Control I2C
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
  
#include  "i2c.h"
#include  "fram.h"  

/*
Byte I2C interface is quite simple: After setting up the component and starting it you use

 

    I2C_MasterSendStart(DeviceAddress,I2C_WRITE_XFER_MODE);    // Initialize a transaction for writing

    I2C_MasterWriteByte(Register);                // Indicate which register you want to write to

    I2C_MasterWriteByte(Value);                // Write to register

    I2C_MasterSendStop();                    // End of transaction

 

When you want to read from a device you use (example for reading two bytes

 

    I2C_MasterSendStart(DeviceAddress,I2C_WRITE_XFER_MODE);    // Initialize a transaction for writing

    I2C_MasterWrite(Register);                // Indicate which register you want to write to

    I2C_MasterSendRestart(DeviceAddress,I2C_READ_XFER_MODE);

    I2C_MasterReadByte(I2C_ACK_DATA);            // Read from register

    I2C_MasterReadByte(I2C_NAK_DATA);            // Read from register, last byte is NAKed

    I2C_MasterSendStop();                    // End of transaction

    */

/**
  * @brief  Read some N registers from I2C device
  * @param  slaveAddress - address of the I2C device being queried
  * @param  regAddr     - register address
  * @param  rdData      - pointer to buffer for read
  * @param  count       - count of bytes to transfer
  * @retval             - return value of underlying read operation
  */
uint32_t i2c_read_registers( uint8_t slaveAddress, uint8_t regAddr, uint8_t *rdData, uint32_t count){
  uint32_t  status = TRANSFER_ERROR;

    status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
    if( 4 == status){
      status = I2CM_I2CMasterSendStop( I2C_DELAY);
      if( 0 == status){
        I2CM_I2CMasterClearStatus();
      }
    }
    if( 0 == status){
      status = I2CM_I2CMasterWriteByte(regAddr, I2C_DELAY);
      if( 0 == status){
        status = I2CM_I2CMasterSendRestart( slaveAddress, I2CM_I2C_READ_XFER_MODE, I2C_DELAY);
        if( 0 == status){
         for( uint32_t i = 0; i < count-1; i++){
            if( I2CM_I2CMasterReadByte(I2CM_I2C_ACK_DATA, &rdData[i], I2C_DELAY)){
              status = TRANSFER_ERROR;
              I2CM_I2CMasterSendStop(I2C_DELAY);
              return status;
            }
         }
        if( I2CM_I2CMasterReadByte( I2CM_I2C_NAK_DATA, &rdData[count-1], I2C_DELAY)){
              status = TRANSFER_ERROR;
              I2CM_I2CMasterSendStop(I2C_DELAY);
              return status;
        }
        status = I2CM_I2CMasterSendStop( I2C_DELAY);
      }
    }
  }
  return status;
}
//  TODO:
#if 0
/**
  * @brief  Send preamble register address to RTC
  * @param  slaveAddress  - address of RTC
  * @param  regAddr       - address of register
  * @retval status
  */
int32_t i2c_write_prefix( uint8_t slaveAddress, uint8_t regAddr){
    uint32_t status = TRANSFER_ERROR;
    I2CM_I2CMasterClearStatus();
    if( I2CM_I2C_MSTR_NO_ERROR == I2CM_I2CMasterWriteBuf( slaveAddress, &regAddr, 1, I2CM_I2C_MODE_COMPLETE_XFER)){
      //  Wait for completion
      while (0u == (I2CM_I2CMasterStatus() & I2CM_I2C_MSTAT_WR_CMPLT))
      {
              /* Wait */
      }
      /* Display transfer status */
      if (0u == (I2CM_I2CMasterStatus() & I2CM_I2C_MSTAT_ERR_XFER))
      {
        /* Check if all bytes was written */
        if (I2CM_I2CMasterGetWriteBufSize() == 1)
        {
            status = TRANSFER_CMPLT;
        }
      }    
    }
    return status;
}
#endif  //  TODO:
#if 0
/**
  * @brief  Write data to FRAM
  * @param  slaveAddress - address of the bridge
  * @param  command - pointer to bytes to send
  * @param  count - number of bytes to send
  * @retval status of operation
  * Sequence <Start><opcode><data0>...<datan><Stop>
  */
uint32_t i2c_write_fram( uint8_t slaveAddress, uint8_t *command, uint32_t count){
  uint32_t status = TRANSFER_ERROR;
  uint32_t   opcode = SPI_READ_WRITE;

  I2CM_I2CMasterClearStatus();
  status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
  if( 0 == status){
    status = I2CM_I2CMasterWriteByte( opcode, I2C_DELAY);
    if( 0 == status){
      for( uint32_t i = 0; i < count; i++){
        status = I2CM_I2CMasterWriteByte(command[i], I2C_DELAY);
        if( status){
          I2CM_I2CMasterSendStop( I2C_DELAY);
          return status;
        }
      }
    }
    status = I2CM_I2CMasterSendStop( I2C_DELAY);
  }
  return status;
}
#endif
/**
  * @brief  Write query to FRAM
  * @param  slaveAddress - address of the bridge
  * @param  command - pointer to bytes to send
  * @param  count - number of bytes to send
  * @retval status of operation
  * Sequence <Start><opcode><data0>...<datan>(optional)<Stop>
  */
uint32_t i2c_query_fram( uint8_t slaveAddress, uint8_t *command, uint32_t count){
  uint32_t status = TRANSFER_ERROR;
  uint32_t   opcode = SPI_READ_WRITE;

  I2CM_I2CMasterClearStatus();
  status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
  if( 0 == status){
    status = I2CM_I2CMasterWriteByte( opcode, I2C_DELAY);
    if( 0 == status){
      for( uint32_t i = 0; i < count; i++){
        status = I2CM_I2CMasterWriteByte(command[i], I2C_DELAY);
        if( status){
          I2CM_I2CMasterSendStop( I2C_DELAY);
          return status;
        }
      }
    }
  }
  status = I2CM_I2CMasterSendStop( I2C_DELAY);
  return status;
}

/**
  * @brief  Read response from FRAM
  * @param  slaveAddress - address of the bridge
  * @param  response - pointer to reply buffer
  * @param  count - number of bytes expected in reply
  * @retval status of operation
  * @note   Assumes a command has been sent via i2c_query_fram()
  */
uint32_t i2c_read_fram( uint8_t slaveAddress, uint8_t *reply, uint32_t count){
  uint32_t status;
  
  I2CM_I2CMasterClearStatus();
  status = I2CM_I2CMasterSendStart( slaveAddress, I2CM_I2C_READ_XFER_MODE, I2C_DELAY);
  if( 0 == status){
    for( uint32_t i = 0; i < count-1; i++){
      status = I2CM_I2CMasterReadByte(I2CM_I2C_ACK_DATA, &reply[i], I2C_DELAY);
      if( status){
        I2CM_I2CMasterSendStop( I2C_DELAY);
        return status;
      }
    }
    status = I2CM_I2CMasterReadByte( I2CM_I2C_NAK_DATA, &reply[count-1], I2C_DELAY);
    if( 0 == status){
        I2CM_I2CMasterSendStop( I2C_DELAY);
        return status;
    }
    status = I2CM_I2CMasterSendStop( I2C_DELAY);
  }
  return status;
}
/**
  * @brief  Write registers to RTC
  * @param  slaveAddress - address of the I2C device being queried
  * @param  regAddr      - register in RTC to read
  * @param  rdData      - pointer to data for write
  * @param  count       - count of bytes to transfer
  * @retval             - return value of underlying write operation
  */
uint32_t i2c_write_registers( uint8_t slaveAddress, uint8_t regAddr, uint8_t *wrData, uint32_t count){
  uint32_t  status = TRANSFER_ERROR;
  
  I2CM_I2CMasterClearStatus();
  status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
  if( 0 == status){
    status = I2CM_I2CMasterWriteByte(regAddr, I2C_DELAY);
      if( 0 == status){
       for( uint32_t i = 0; i < count; i++){
          if( I2CM_I2CMasterWriteByte(wrData[i], I2C_DELAY)){
            status = TRANSFER_ERROR;
            I2CM_I2CMasterSendStop(I2C_DELAY);
          } 
        }
      status = I2CM_I2CMasterSendStop( I2C_DELAY);
    }
  }
  return status;
}
/**
  * @brief  Read memory from FRAM via bridge
  * @param  slaveAddress - address of the I2C device being queried
  * @param  memAddress   - address in the FRAM to write
  * @param  rdData      - pointer to buffer for read
  * @param  count       - count of bytes to transfer
  * @retval             - return value of underlying read operation
  */
uint32_t i2c_read_memory( uint8_t slaveAddress, uint32_t memAddress, uint8_t *rdData, uint32_t count){
  uint32_t  status = TRANSFER_ERROR;
  uint32_t   opcode = SPI_READ_WRITE;
  uint8_t  temp;
  
  I2CM_I2CMasterClearStatus();

  status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
  if( 0 == status){
    status = I2CM_I2CMasterWriteByte( opcode, I2C_DELAY);
    if( 0 == status){
      status = I2CM_I2CMasterWriteByte(memAddress >> 8, I2C_DELAY);
      if( 0 == status){
        status = I2CM_I2CMasterWriteByte( memAddress & 0xFF, I2C_DELAY);
        if( 0 == status){
          status = I2CM_I2CMasterSendRestart( slaveAddress, I2CM_I2C_READ_XFER_MODE, I2C_DELAY);
          if( 0 == status){
            for( uint32_t i = 0; i < count-1; i++){
              //if( I2CM_I2CMasterReadByte(I2CM_I2C_ACK_DATA, &rdData[i], I2C_DELAY)){
              if( I2CM_I2CMasterReadByte(I2CM_I2C_ACK_DATA, &temp, I2C_DELAY)){
                status = TRANSFER_ERROR;
                I2CM_I2CMasterSendStop(I2C_DELAY);
                return status;
              }
              else
                rdData[i] = temp;
            }
            if( I2CM_I2CMasterReadByte( I2CM_I2C_NAK_DATA, &rdData[count-1], I2C_DELAY)){
              status = TRANSFER_ERROR;
              I2CM_I2CMasterSendStop(I2C_DELAY);
              return status;
            }
          }
        }
        status = I2CM_I2CMasterSendStop( I2C_DELAY);
      }
    }
  }
  
  return status;
}

/**
  * @brief  Write memory to FRAM via bridge
  * @param  slaveAddress - address of the I2C device being queried
  * @param  rdData      - pointer to data for write
  * @param  count       - count of bytes to transfer
  * @retval             - return value of underlying write operation
  */
uint32_t i2c_write_memory( uint8_t slaveAddress, uint32_t memAddr, uint8_t *wrData, uint32_t count){
  uint32_t  status = TRANSFER_ERROR;
  uint8_t   opcode = SPI_READ_WRITE;
  
  I2CM_I2CMasterClearStatus();
  status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
  if( 0 == status){
    status = I2CM_I2CMasterWriteByte(opcode, I2C_DELAY);
    if( 0 == status){
      status = I2CM_I2CMasterWriteByte(memAddr >> 8, I2C_DELAY);
        if( 0 == status){
          status = I2CM_I2CMasterWriteByte(  memAddr & 0xFF, I2C_DELAY);
          if( 0 == status){
            for( uint32_t i = 0; i < count; i++){
              if( I2CM_I2CMasterWriteByte(wrData[i], I2C_DELAY)){
                I2CM_I2CMasterSendStop(I2C_DELAY);
                return TRANSFER_ERROR;
              }
          }
          status = I2CM_I2CMasterSendStop( I2C_DELAY);
        }
      }
    }
  }
  return status;
}
/**
  * @brief  Read from the bridge device
  * @param  slaveAddress - address of the I2C device being queried
  * @param  rdData      - pointer to buffer for read
  * @param  count       - count of bytes to transfer
  * @retval             - return value of underlying read operation
  * @note   **** Assumes a read command has already been sent ****
  */
uint32_t i2c_read_bridge( uint8_t slaveAddress, uint8_t *rdData, uint32_t count){
  uint32_t  status = TRANSFER_ERROR;

//  status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
//  if( 0 == status){
    status = I2CM_I2CMasterSendRestart( slaveAddress, I2CM_I2C_READ_XFER_MODE, I2C_DELAY);
    if( 0 == status){
      for( uint32_t i = 0; i < count-1; i++){
        if( I2CM_I2CMasterReadByte(I2CM_I2C_ACK_DATA, &rdData[i], I2C_DELAY)){
          status = TRANSFER_ERROR;
          I2CM_I2CMasterSendStop(I2C_DELAY);
          return status;
        }
      }
      if( I2CM_I2CMasterReadByte( I2CM_I2C_NAK_DATA, &rdData[count-1], I2C_DELAY)){
        status = TRANSFER_ERROR;
        I2CM_I2CMasterSendStop(I2C_DELAY);
        return status;
      }
    }
    status = I2CM_I2CMasterSendStop( I2C_DELAY);
//  }
  return status;
}

/**
  * @brief  Write to bridge
  * @param  slaveAddress - address of the I2C device being queried
  * @param  rdData      - pointer to data for write
  * @param  count       - count of bytes to transfer
  * @retval             - return value of underlying write operation
  */
uint32_t i2c_write_bridge( uint8_t slaveAddress, uint8_t *wrData, uint32_t count){
  uint32_t  status = TRANSFER_ERROR;
  
  I2CM_I2CMasterClearStatus();
  status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
  if( 0 == status){
    for( uint32_t i = 0; i < count; i++){
      if( I2CM_I2CMasterWriteByte(wrData[i], I2C_DELAY)){
        I2CM_I2CMasterSendStop(I2C_DELAY);
        return TRANSFER_ERROR;
      }
    }
    status = I2CM_I2CMasterSendStop( I2C_DELAY);
  }
  return status;
}
/**
  * @brief  Write to bridge
  * @param  slaveAddress - address of the I2C device being queried
  * @param  rdData      - pointer to data for write
  * @param  count       - count of bytes to transfer
  * @retval             - return value of underlying write operation
  */
uint32_t i2c_query_bridge( uint8_t slaveAddress, uint8_t *wrData, uint32_t count){
  uint32_t  status = TRANSFER_ERROR;
  
  I2CM_I2CMasterClearStatus();
  status = I2CM_I2CMasterSendStart(slaveAddress,I2CM_I2C_WRITE_XFER_MODE, I2C_DELAY);
  if( 0 == status){
    for( uint32_t i = 0; i < count; i++){
      if( I2CM_I2CMasterWriteByte(wrData[i], I2C_DELAY)){
        I2CM_I2CMasterSendStop(I2C_DELAY);
        return TRANSFER_ERROR;
      }
    }
//    status = I2CM_I2CMasterSendStop( I2C_DELAY);
  }
  return status;
}

/* [] END OF FILE */
