/**
  ******************************************************************************
  * @file    	keyboard.c
  * @brief   	Manages two input buttons
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
 
#include  "keyboard.h"
#include  "buttons.h"
#include  "UB_1.h"
#include  "UB_2.h"
#include  "timebase.h"

/** @note Table of button control variables
  */
button_t  buttons[ NUM_BUTTONS];
/** @note Table of functions to initialize the buttons[] table
  */
button_functions_t buttonFuncs[ NUM_BUTTONS] = 
{
  { &powerButtonAction, &UB_1_Read },
  { &coolingButtonAction, &UB_2_Read}
  
};
/**
	*	@brief	This function reads a button, determines its state, and determines whether it's been in that state
	*					long enough to consider it a key closure. If so, on the release, call the action function with the 
  *         duration of the keypress.
	*	@param	Pointer to a button structure describing the button to debounce
	*	@retval	None - buttons are update in situ
	*/
int32_t debounce_button( button_t *b){
  
  //  What is the current state? (Pressed is 0)
	int8_t sample = b->pRead();
	  
  //  If not the same as last read, reset debounce time and sample
  if( sample){  //  pulled high - it's open
    if( !b->lastRead){   //  was pressed last time
      b->lastRead = sample;
      b->depressDuration = now() - b->debounceStart;
      b->pAction( b->depressDuration);
    }
  }
  else{         //  key pressed
    if( b->lastRead){  //  and not pressed last time
      b->debounceStart = now();
      b->lastRead = sample;
    }
  }
  return 0;
}

int32_t process_keyboard( void){
  for( int i = 0; i < NUM_BUTTONS; i++){
    debounce_button( &buttons[ i]);
  }
  return 0;
}

int32_t init_keyboard( void){
  for( int i = 0; i < NUM_BUTTONS; i++){
    buttons[i].pRead = buttonFuncs[ i].pRead;
    buttons[i].pAction = buttonFuncs[ i].pAction;
    buttons[i].lastRead = 1;
  }
  return 0;
}
  
  /* [] END OF FILE */
