/**
  ******************************************************************************
  * @file    	logging.c
  * @brief   	Maintain logging for Cooling Socket
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#include  "logging.h"
#include  "rtc.h"
#include  "fram.h"
/**
  * @brief  Format a log entry
  * @param  Time - log time
  * @param  tHot -  hot side temperature
  * @param  tCold - cold side temperature
  * @param  tSkin - synthesized skin temperature
  * @param  vTec  - voltage on the TEC control pin
  * @param  iTec  - current through the TEC
  * @param  vBatt - Batter voltage
  * @param  iFan  - current through the fan
  * @param  COP   - Coefficient of Performance?
  * @retval 0 for success
  * @note   This platform does not have a sprintf or near equivalent, so we'll use some brute force...
  * Log string has the following outline:
  * 000000000011111111112222222222333333333344444444445555555555666666666677777777778888888888
  * 012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
  * YYYY:MM:DD:hh:mm:ss.uu,thot,tcol,tskn,vtec,itec,vbat,ifan,cccccccc\n
  * 
  */
log_print_t print_points[]=
{
  {3, 2},   //  YYYY
  {6, 2},   //  MM
  {9, 2},   //  DD
  {12, 2},  //  hh
  {15, 2},  //  mm
  {18, 2},  //  ss
  {21, 2},  //  uu
  {26 , 4}, //  thot
  {31, 4},  //  tcol
  {36, 4},  //  tskn
  {41, 4},  //  vtec
  {46, 4},  //  itec
  {51, 4},  //  vbat
  {56, 4},  //  ifan
  {65, 8}   //  COP
};

char *logStringTemplate="20YY-MM-DD,hh:mm:ss.uu,0000,0000,0000,0000,0000,0000,0000,00000000\n\r";
char *errorStringTemplate="20YY-MM-DD,hh:mm:ss.uu  ";
/**
  * log_buff is used to hold strings going to and from the FRAM
  * string_buf is used to hold strings going into log_buff
  */
uint8_t log_buff[ FRAM_PAGE_SIZE];
uint8_t string_buff[ LOG_BUFLEN];

/**
  * @brief  Clear a buffer to FF
  * @param  buffer - pointer to area to clear
  * @param  size - number of bytes to clear
  * @retval none
  */
void     erase_buff( uint8_t *buff, uint32_t size){
  if( size & 3){  //  By the byte
    for( uint32_t i = 0; i < size; i++){
      buff[i] = 0xFF;
    }
  }
  else if( size & 1){ //  By the word
    uint16_t *p = (uint16_t *)buff;
    for( uint32_t i = 0; i < size/2; i++){
      p[i] = 0xFFFF;
    }
  }
  else{ //  By the dword
    uint32_t *p = (uint32_t *)buff;
    for( uint32_t i = 0; i < size/4; i++){
      p[i] = 0xFFFFFFFF;
    }
  }
}
/**
  * @brief  Initialize the logging system
  * @param  None
  * @retval None
  */
void log_init( void){
  logvars_t temp;
  rtc_read_logvars( RTC_ADDR, &temp);
  fram_read( FRAM_ADDR, log_buff, logVars.activeLogPage, FRAM_PAGE_SIZE);
}
  
/**
  * @brief  Convert a decimal digit ('0'..'9') to ASCII
  * @param  digit to convert
  * @return converted digit
  * @note   This routine is included for clarity (rather than just adding '0' randomly)
  */
char  todigit( int16_t x){
  return x + '0';
}

/**
  * @brief  - convert a hex digit to ASCII
  * @param  tohex - digit to convert
  * @retval Ascii digit
  */
uint8_t unhex[16]="0123456789ABCDEF";
uint8_t hexof( uint8_t tohex){
  CYASSERT( tohex < 16);
  return unhex[tohex];
}

/**
  * @brief  Print one field in the log string
  * @param  Buffer - pointer to string
  * @param  index - where in the string to start writing "backward"
  * @param  length - number of digits to convert
  * @param  value - value to print
  * @note   Prints values backward, from the least-signficant digit to the most-significant
  * @retval 0 - always successful.
*/
int16_t printplaces(  uint8_t *buffer, int16_t index, int16_t length, int32_t value){
  for( int i = 0; i < length; i++){
    buffer[index--] = todigit( value % 10);
    value /= 10;
  }
  return 0;
}
typedef enum{
  YR = 0,
  MO = 1,
  DY = 2,
  HR = 3,
  MN = 4,
  SC = 5,
  US = 6,
  TH = 7,
  TC = 8,
  TS = 9,
  VT = 10,
  IT = 11,
  VB = 12,
  FI = 13,
  CP = 14
}print_point_index_t;
/**
  * @brief  Build and write a log message
  * @param  Should be self-explanatory, but...
  * @param  year
  * @param  month
  * @param  day
  * @param  hour
  * @param  minutes
  * @param  seconds
  * @param  cSec  - RTC resolution is to 0.01 seconds, so report centiSeconds
  * @param  tHot  - measured hot side of TEC
  * @param  tCold - measured cold side of TEC
  * @param  tSkin - CALCULATED skin temperature
  * @param  vTec  - CALCULATED control voltage to TEC
  * @param  iTec  - measured TEC current
  * @param  vBatt - measured battery voltage
  * @param  iFan; - measured fan current
  * @param  COP   - CALCULATED "Coefficient of Performance"
  * @retval 0 - always success!
  * @note   Makes use of a template string (logStringTemplate) and a lookup table
  *         of indices and lengths of variables to print "backward" using printplaces()
  */
uint32_t log_measurements(  int32_t year, int16_t month, int16_t day, int16_t hour, int16_t minutes,int16_t seconds, int16_t cSec, int16_t tHot, int16_t tCold, int16_t tSkin, int16_t vTec, int16_t iTec, int16_t vBatt, int16_t iFan, int32_t COP){
  CYASSERT( strlen( logStringTemplate) < LOG_BUFLEN);
  strcpy( (char *)string_buff, logStringTemplate);
  printplaces( string_buff, print_points[YR].index, print_points[YR].length, (int32_t)year);
  printplaces( string_buff, print_points[MO].index, print_points[MO].length, (int32_t)month);
  printplaces( string_buff, print_points[DY].index, print_points[DY].length, (int32_t)day);
  printplaces( string_buff, print_points[HR].index, print_points[HR].length, (int32_t)hour);
  printplaces( string_buff, print_points[MN].index, print_points[MN].length, (int32_t)minutes);
  printplaces( string_buff, print_points[SC].index, print_points[SC].length, (int32_t)seconds);
  printplaces( string_buff, print_points[US].index, print_points[US].length, (int32_t)cSec);
  printplaces( string_buff, print_points[TH].index, print_points[TH].length, (int32_t)tHot);
  printplaces( string_buff, print_points[TC].index, print_points[TC].length, (int32_t)tCold);
  printplaces( string_buff, print_points[TS].index, print_points[TS].length, (int32_t)tSkin);
  printplaces( string_buff, print_points[VT].index, print_points[VT].length, (int32_t)vTec);
  printplaces( string_buff, print_points[IT].index, print_points[IT].length, (int32_t)iTec);
  printplaces( string_buff, print_points[VB].index, print_points[VB].length, (int32_t)vBatt);
  printplaces( string_buff, print_points[FI].index, print_points[FI].length, (int32_t)iFan);
  printplaces( string_buff, print_points[CP].index, print_points[CP].length, (int32_t)COP);
  strcat( (char *)string_buff, "\n\r");
  //UART_UartPutString( (const char *)log_buff);
  log_store( (const char *)string_buff);
  return 0;
}
/**
  * @brief  - Write a string to the log file
  * @param  - Pointer to string to be written
  * @return - Number of characters written
  */
uint32_t log_string( int32_t year, int16_t month, int16_t day, int16_t hour, int16_t minutes,int16_t seconds, int16_t cSec, const char *str){
  CYASSERT( strlen( errorStringTemplate) < LOG_BUFLEN);
  uint32_t usedlen = strlen( errorStringTemplate) + strlen( "\n\r");  //  Passed string goes between template and \n\r
  uint32_t freelen = LOG_BUFLEN - usedlen;                    //
  strcpy( (char *)string_buff, errorStringTemplate);
  printplaces( string_buff, print_points[YR].index, print_points[YR].length, (int32_t)year);
  printplaces( string_buff, print_points[MO].index, print_points[MO].length, (int32_t)month);
  printplaces( string_buff, print_points[DY].index, print_points[DY].length, (int32_t)day);
  printplaces( string_buff, print_points[HR].index, print_points[HR].length, (int32_t)hour);
  printplaces( string_buff, print_points[MN].index, print_points[MN].length, (int32_t)minutes);
  printplaces( string_buff, print_points[SC].index, print_points[SC].length, (int32_t)seconds);
  printplaces( string_buff, print_points[US].index, print_points[US].length, (int32_t)cSec);
  //  If the incoming string is too long to buffer, truncate the string. It ought never be...
  if(strlen(str) > freelen){
    strncat( (char *)string_buff, str, freelen);
  }
  else{
    strcat( (char *)string_buff, str);
  }
  strcat( (char *)string_buff, "\n\r");
  
  if( 0 == uartSuspended){
    UART1_UartPutString( (const char *)log_buff);
  }
  log_store( (const char *)string_buff);
  return 0;
}
/**
  * @brief  - Write a string to the log file
  * @param  - Pointer to string to be written
  * @return - Number of characters written
  */
uint32_t print_string( int32_t year, int16_t month, int16_t day, int16_t hour, int16_t minutes,int16_t seconds, int16_t cSec, const char *str){
  uint32_t usedlen = strlen( errorStringTemplate) + strlen( "\n\r");  //  Passed string goes between template and \n\r
  uint32_t freelen = LOG_BUFLEN - usedlen;                    //
  strcpy( (char *)string_buff, errorStringTemplate);
  printplaces( string_buff, print_points[YR].index, print_points[YR].length, (int32_t)year);
  printplaces( string_buff, print_points[MO].index, print_points[MO].length, (int32_t)month);
  printplaces( string_buff, print_points[DY].index, print_points[DY].length, (int32_t)day);
  printplaces( string_buff, print_points[HR].index, print_points[HR].length, (int32_t)hour);
  printplaces( string_buff, print_points[MN].index, print_points[MN].length, (int32_t)minutes);
  printplaces( string_buff, print_points[SC].index, print_points[SC].length, (int32_t)seconds);
  printplaces( string_buff, print_points[US].index, print_points[US].length, (int32_t)cSec);
  //  If the incoming string is too long to buffer, truncate the string. It ought never be...
  if(strlen(str) > freelen){
    strncat( (char *)string_buff, str, freelen);
  }
  else{
    strcat( (char *)string_buff, str);
  }
  strcat( (char *)string_buff, "\n\r");
  if( 0 == uartSuspended){
    UART1_UartPutString( (const char *)string_buff);
  }
  return 0;
}

/**
  * @brief  Log an error string
  * @param  String to log
  * @retval 0
  * @note   Fetch time, call log_string();
  */
uint32_t log_error( const char *str){
  rtc_time_t localTime;
  rtc_read_clock( RTC_ADDR, &localTime);
  log_string( localTime.year, localTime.month, localTime.date, localTime.hrs, localTime.mins,
              localTime.secs, localTime.hund, (const char *)str);
  return 0;
}

/**
  * @brief  print an error string to the console
  * @param  String to print
  * @retval 0
  * @note   Fetch time, print
  */
uint32_t print_error( const char *str){
  rtc_time_t localTime;
  rtc_read_clock( RTC_ADDR, &localTime);
  print_string( localTime.year, localTime.month, localTime.date, localTime.hrs, localTime.mins,
              localTime.secs, localTime.hund, (const char *)str);
  return 0;
}
/**
  * @note The FRAM is comprised of 
  *   1024 blocks (of)
  *     64 pages  (of)
  *   2048 bytes
  */
typedef struct{
  uint32_t  index;    //  Where in the page (0..2047)
  uint32_t  page;     //  Which page? (0..63)
  uint32_t  block;    //  Which block? (0..1023)
}fram_track_t;
/**
  * @brief  - write a log string to FRAM
  * @param  - str - pointer to string for logging
  * @retval - status of operation
  * @note   - Always write buffer to FRAM. If the buffer fills,
  *         - write it and start a new page, also written to FRAM.
  * @note   - Also read and save logging variables in RTC
  */
uint32_t  log_store( const char *str){
  uint32_t status = 0;
  uint32_t remainder = 0;
  uint32_t toCopy;
  int32_t buff_end;
  /**
    * @note - grab the backed up vars
    */
  rtc_read_logvars( RTC_ADDR, &logVars);
  //CYASSERT( logVars.currentPageIndex < FRAM_PAGE_SIZE);
  buff_end = logVars.currentPageIndex + strlen( str);
  /*
    * @note check if the buffer overflows with the incoming string
    */
  if( buff_end > FRAM_PAGE_SIZE){
    remainder = buff_end - FRAM_PAGE_SIZE;
    toCopy = FRAM_PAGE_SIZE - logVars.currentPageIndex;
    strncpy( (char *)&log_buff[logVars.currentPageIndex], str, toCopy);
  }
  else{
    strcpy( (char *)&log_buff[logVars.currentPageIndex], str);
    logVars.currentPageIndex += strlen( str);
  }
#if 0  // -- no logging until I figure out FRAM for real
  fram_write( FRAM_ADDR,(uint8_t *) log_buff, logVars.activeLogPage, FRAM_PAGE_SIZE);
  if( remainder){
    logVars.activeLogPage++;  
    logVars.currentPageIndex = 0;
    erase_buff( log_buff, FRAM_PAGE_SIZE);
    strcpy( (char *)log_buff, (char *)&str[toCopy]); 
    fram_write( FRAM_ADDR, (uint8_t *)log_buff, logVars.activeLogPage, FRAM_PAGE_SIZE);
  }
#endif  
  /**
    * @note - save the backed up vars
    */
  rtc_write_logvars( RTC_ADDR, &logVars);
  return status;
}
/* [] END OF FILE */
