/**
  ******************************************************************************
  * @file    	rtc.c
  * @brief   	Control real time clock 
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#include  "rtc.h"
#include  "fram.h"

sysvars_t sysVars;
logvars_t logVars;

/**
  * @brief  Convert a BCD to binary
  * @param  bcd is a BCD digit
  * @retval binary equivalent
  * @note   BCD (e.g. 38) 
  *         ((38 >> 4) & 0xF) = 3
  *         3 * 10 = 30
  *         38  & 0xF = 8
  *         + 30 = 38.
  */
uint8_t bcd_to_binary( uint8_t bcd){
  return ( (((bcd >> 4) & 0xF) * 10) + ( bcd & 0xF));
}
/**
  * @brief  Convert a binary to a BCD
  * @param  binary - binary integer coming in
  * @retval BCD equivalent or 0xFF
  * @note   binary (eg 0x38 == 56)
  *         x38 mod 10 = 5
  *         x32 / 10 = 6
  *         bcd = (bcd/10)<< 4 + bcd % 10
  */
uint8_t binary_to_bcd( uint8_t bin){
  return (( bin / 10) << 4) + bin%10;
}
/**
  * @brief  generate a simple XOR checksum on the data buffer
  * @param  chkBuffer - pointer to "uint8_t" buffer
  * @param  chkSize - length of checksum area
  * @retval status of operation verification
  * @note   Checksum is performed on chkSize - 1 bytes
  * @note   Checksum is placed in location chkBuffer[chkSize-1]
  */
int32_t gen_checksum( uint8_t *chkBuffer, uint32_t chkSize){
  int8_t chk = 0;
  for( uint32_t i = 0; i < chkSize -1; i++){
    chk ^= chkBuffer[ i];
  }
  chkBuffer[chkSize - 1] = chk;
  return (int32_t)match_checksum( chkBuffer, chkSize);
}
/**
  * @brief  validate a simple XOR checksum on the data buffer
  * @param  chkBuffer - pointer to "uint8t" buffer
  * @param  chkSize - length of checksum area
  * @retval Whether the checksum is good (0) or bad (1)
  * @note   Checksum is performed on all chkSize bytes of the buffer;
  * @note   A correct checksum will add up to zero
  */
int32_t match_checksum( uint8_t *chkBuffer, uint32_t chkSize){
  int8_t chk = 0;
  for( uint32_t i = 0; i < chkSize; i++){
    chk ^= chkBuffer[ i];
  }
  return (int32_t)chk;
}

/**
  * @brief  Read some N registers from the Real Time Clock
  * @param  slaveAddress - address of the I2C device being queried
  * @param  regAddr     - starting register
  * @param  rdData      - pointer to buffer for read
  * @param  count       - count of bytes to transfer
  * @retval             - return value of underlying read operation
  */
uint32_t rtc_read_registers( uint8_t slaveAddress, uint8_t regAddr, uint8_t *rdData, uint32_t count){
  return i2c_read_registers( slaveAddress, regAddr, rdData, count);
}

/**
  * @brief  Write some N registers to the Real Time Clock
  * @param  slaveAddress - address of the I2C device being queried
  * @param  regAddr     - starting register
  * @param  rdData      - pointer to data for write
  * @param  count       - count of bytes to transfer
  * @retval             - return value of underlying write operation
  */
uint32_t rtc_write_registers( uint8_t slaveAddress, uint8_t regAddr, uint8_t *wrData, uint32_t count){
  return i2c_write_registers( slaveAddress, regAddr, wrData, count);
}

/**
  * @brief  Read the RTC to get current time, then convert from BCD to binary
  * @param  slaveAddress - address of the I2C clock
  * @param  rdData - pointer to time structure
  * @retval status of the operation
  */
uint32_t rtc_read_clock( uint8_t slaveAddress, rtc_time_t *rdData){
  uint32_t  status;
  rtc_time_t bcd;
  uint8_t *in, *out;
  status = rtc_read_registers( slaveAddress, HUND_REG, (uint8_t*)&bcd, sizeof( bcd));
    if( 0 == status){
    /*  being mindful that both structures contain only unsigned bytes...
    */
    in = (uint8_t *)&bcd;
    out = (uint8_t *)rdData;
    
    for( uint32_t i = 0; i < sizeof(bcd); i++){
      out[i] = bcd_to_binary( in[i]);
    }
  }
  return status;
}

uint32_t rtc_write_clock( uint8_t slaveAddress, rtc_time_t *wrData){
  uint32_t  status;
  rtc_time_t bcd;
  uint8_t *in, *out;
  /*  being mindful that both structures contain only unsigned bytes...
  */
  in = (uint8_t *)wrData;
  out = (uint8_t *)&bcd;

  for( uint32_t i = 0; i < sizeof( bcd); i++){
    out[i] = binary_to_bcd( in[i]);
  }
  status = rtc_write_registers( slaveAddress, HUND_REG, (uint8_t *)&bcd, sizeof( bcd));
  
  return status;
}
/**
  * @brief  Read system variables from the RTC's memory
  * @param  slaveAddress - I2C address of RTC
  * @param  rdData - pointer to return buffer
  * @retval status of operation
  */
uint32_t rtc_read_sysvars(uint8_t slaveAddress, sysvars_t *rdData){
  int32_t status;
  status = rtc_read_registers( slaveAddress, SYSTEM_VARS_LOC, (uint8_t*) rdData, sizeof( sysvars_t));
  if( 0 == status){
    status = match_checksum( (uint8_t*)rdData, sizeof( sysvars_t));
  }
  return status;
}
/**
  * @brief  Write system variables to the RTC's memory
  * @param  slaveAddress - I2C address of RTC
  * @param  wrData - pointer to data to be saved
  * @retval status of operation
  */
uint32_t rtc_write_sysvars(uint8_t slaveAddress, sysvars_t *wrData){
  int32_t status;
  status = rtc_write_registers( slaveAddress, SYSTEM_VARS_LOC, (uint8_t*) wrData, sizeof( sysvars_t));
  return status;
}

/**
  * @brief  Read system variables from the RTC's memory
  * @param  slaveAddress - I2C address of RTC
  * @param  rdData - pointer to return buffer
  * @retval status of operation
  */
uint32_t rtc_read_logvars(uint8_t slaveAddress, logvars_t *rdData){
  logvars_t temp;
  rtc_read_registers( slaveAddress, LOG_VARS_LOC, (uint8_t*) &temp, sizeof( logvars_t));
  //  Check for the unusual case where the data is all FF...
  if(( temp.activeLogPage == 0xFFFFFFFF) && ( temp.currentPageIndex = 0xFFFF) ){
    temp.activeLogPage = 55; // Just break it
  }
  if( 0 == match_checksum( (uint8_t *)&temp, sizeof( logvars_t))){
    memcpy( (uint8_t *)rdData, (uint8_t *)&temp, sizeof( logvars_t));
  }
  else{
    /** TODO: Realistically, we need to find the last page written, as well as where in the last page
      * we've written to, but that's an exercise left for the reader...
      */
    temp.currentPageIndex = 0;
    temp.activeLogPage = 0;
    gen_checksum( (uint8_t *)&temp, sizeof( logvars_t));
    rtc_write_logvars( RTC_ADDR, &temp);
    memcpy( (uint8_t *)rdData, (uint8_t *)&temp, sizeof( logvars_t));
  }
  
  return 0;
}
/**
  * @brief  Write system variables to the RTC's memory
  * @param  slaveAddress - I2C address of RTC
  * @param  wrData - pointer to data to be saved
  * @retval status of operation
  */
uint32_t rtc_write_logvars(uint8_t slaveAddress, logvars_t *wrData){
  int32_t status;
  CYASSERT( wrData->currentPageIndex < FRAM_PAGE_SIZE);

  gen_checksum( (uint8_t *)wrData, sizeof( logvars_t));
  status = rtc_write_registers( slaveAddress, LOG_VARS_LOC, (uint8_t*) wrData, sizeof( logvars_t));
  return status;
}
/**
  * @brief  Get the value of the trickle charge register
  * @param  slaveAddress - I2C address of the RTC
  * @param  trickle     - pointer to return storage
  * @retval status of the operation
  */
uint32_t rtc_get_trickle( uint8_t slaveAddress, uint8_t *trickle){
  uint32_t status = rtc_read_registers( slaveAddress, TRIK_REG, trickle, 1);
  return status;
}

/**
  * @brief  Set the value of the trickle charge register
  * @param  slaveAddress - I2C address of the RTC
  * @param  trickle     - value to program
  * @retval status of the operation
  */
uint32_t rtc_set_trickle( uint8_t slaveAddress, uint8_t trickle){
  uint32_t status;
  
  uint8_t opcode = 0x9D;
  status = rtc_write_registers( slaveAddress, CONF_KEY, &opcode, 1);    //  Magic key to unlock trickle register
  opcode = trickle;
  status = rtc_write_registers( slaveAddress, TRIK_REG, &opcode, 1);
  opcode = 0;
  status = rtc_write_registers( slaveAddress, CONF_KEY, &opcode, 1);    //  Writing zero resets register
  
  return status;
}

/* [] END OF FILE */
