/**
  ******************************************************************************
  * @file    	rtc.h
  * @brief   	Control real time clock 
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#ifndef __RTC_H
#define __RTC_H

#include  "project.h"
#include  "I2CM.h"
#include  "i2c.h"
  
//#define   RTC_ADDR    0xD2    //  1101001X where X is the read/write bit
#define   RTC_ADDR    0x69      //  110 1001X is the way it really looks
                                //  Code underneath shifts left one bit (0xD2) and
                                //  then adds the Write bit if needed...
#define RTC_RAM_LEN 60          //  It's actually longer (0xC0) but this is what I need
 
#define   TRANSFER_ERROR    1
#define   TRANSFER_CMPLT    0
  
#define SYSTEM_VARS_LOC       0x40         //  Arbitrary register address in RTC 
                                           //  where system variables are stored.
#define LOG_VARS_LOC          0x40 + 0x14  //  Where to save log variables (System vars take 20 bytes. Yeah, I know 
                                           // this is ugly, but it works. TODO:

#define ZERO                  (0UL)  

  /**
    * Register definitions for the RTC.
    * Not what one might consider contiguous...
    */
typedef enum{
  HUND_REG = 0,
  SECS_REG,
  MINS_REG,
  HOUR_REG,
  DATE_REG,
  MNTH_REG,
  YEAR_REG,
  WKDY_REG,
  
  HUND_ALM = 8,
  SECS_ALM,
  HOUR_ALM,
  DATE_ALM,
  MNTH_ALM,
  WKDY_ALM,
  
  STAT_REG = 0xF,
  CTR1_REG,
  CTR2_REG,
  IMSK_REG,
  SQW_REG,
  
  CALXT_REG = 0x14,
  CALRCUPPER = 0x15,
  CALRCLOWER = 0x16,
  SLP_CTRL = 0x17,
  TMR_CTRL,
  TIMER_REG,
  TIMER_INIT,
  WDOG_REG,
  OSC_CTRL,
  OSC_STAT = 0x1D,
//  RESERVED
  CONF_KEY = 0x1F,
  TRIK_REG = 0x20,
  BREF_CTL,
//  RESERVED
//  RESERVED
//  RESERVED
//  RESERVED  
  CAPRC_CTL = 0x26,
  BATMODE,
  ID0_REG,
  ID1_REG,
  ID2_REG,
  ID3_REG,
  ID4_REG,
  ID5_REG,
  ID6_REG,
  ANA_STAT = 0x2F,
  OUTP_CTL,
//  RESERVED
//  RESERVED
//  RESERVED
//  RESERVED  
//  RESERVED
//  RESERVED
//  RESERVED
//  RESERVED  
//  RESERVED
//  RESERVED
//  RESERVED
//  RESERVED  
//  RESERVED
//  RESERVED
//  RESERVED  
  EXTRAM  = 0x3F,
  RTCRAM = 0x40
  
}reg_addr_t;


typedef struct{
  uint8_t hund;
  uint8_t secs;
  uint8_t mins;
  uint8_t hrs;
  uint8_t date;
  uint8_t month;
  uint8_t year;
}rtc_time_t;

/**
  * @note following variables need to be saved...
    T_setPoint - desired skin temperature
    t_Cool - how long to cool
    t_Refractory - how long to rest from cooling
    Rtec - thermal resistance of the TEC
    Rliner - thermal resistance of the liner
    Kconstant - A constant used in calculating skin temperature
  */
typedef struct{
  uint16_t  written;
  uint16_t  setPoint;
  uint16_t  tCool;
  uint16_t  tRefract;
  float32   Rtec;
  float32   Rliner;
  float32   Kconstant;
  int8_t    sysVarChecksum;
}sysvars_t;

/**
  * @note following variables used in tracking the log:
  *   activeLogPage is the page index of the current working page.
  *     It is not necessary to program a whole page - so each logged event is stored to FRAM
  *   currentPageIndex is the offset (0..2047) into the active logging buffer
  */
typedef struct{
  uint32_t activeLogPage;
  uint16_t currentPageIndex;
  uint8_t  logVarsChecksum;
}logvars_t;

extern sysvars_t sysVars;
extern logvars_t logVars;


uint32_t rtc_read_registers( uint8_t slaveAddress, uint8_t regAddr, uint8_t *rdData, uint32_t count);
uint32_t rtc_write_registers( uint8_t slaveAddress, uint8_t regAddr, uint8_t *wrData, uint32_t count);
uint32_t rtc_read_clock( uint8_t slaveAddress, rtc_time_t *rdData);
uint32_t rtc_write_clock( uint8_t slaveAddress, rtc_time_t *wrData);

uint32_t rtc_read_sysvars(uint8_t slaveAddress, sysvars_t *rdData);
uint32_t rtc_write_sysvars(uint8_t slaveAddress, sysvars_t *wrData);
uint32_t rtc_read_logvars(uint8_t slaveAddress, logvars_t *rdData);
uint32_t rtc_write_logvars(uint8_t slaveAddress, logvars_t *wrData);

uint8_t bcd_to_binary( uint8_t bcd);
uint8_t binary_to_bcd( uint8_t bin);

int32_t gen_checksum( uint8_t *chkBuffer, uint32_t chkSize);
int32_t match_checksum( uint8_t *chkBuffer, uint32_t chkSize);
uint32_t rtc_get_trickle( uint8_t slaveAddress, uint8_t *trickle);
uint32_t rtc_set_trickle( uint8_t slaveAddress, uint8_t trickle);

#endif  //  __RTC_H

/* [] END OF FILE */
