/**
  ******************************************************************************
  * @file    	states.c
  * @brief   	State machine
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
#include  "states.h"
  
cooling_state_t currentState = ST_OFF;
const char * eventStr[ 10]={
  "CE_PowerOn\n\r", 
  "CE_PowerOff\n\r", 
  "CE_CoolOn\n\r", 
  "CE_CoolOff\n\r", 
  "CE_Refract\n\r", 
  "CE_Cool\n\r", 
  "CE_IdleTimeout\n\r", 
  "CE_V_Low\n\r", 
  "CE_V_Critical\n\r", 
  "CE_ButtonTap\n\r"
};
const char * stateStr[ 6] = {
  "ST_OFF ", 
  "ST_IDLE ", 
  "ST_COOL ", 
  "ST_REFRACT ", 
  "ST_ERROR ", 
  "ST_PASSIVE "
};
  
/**
  * @brief  Return current machine state
  * @param  none  
  * @retval current state
  */
cooling_state_t getState( void){
  return currentState;
}

/**
  * @brief  Set the current machine state
  * @param  State
  * @retval none
  */
void setState( cooling_state_t s){
  currentState = s;
}
/**
  * @brief  Kick off a 500mS blue LED with a call to the red LED
  * @param  none
  * @retval none
  * @note   lights blue LED and passes pointer to light red LED
  */
void blue_led(){
  start_device_chain( DEV_LED_BLUE, 500, &red_led);
}
/**
  * @brief  Kick off a 500mS red LED with a call to the greem LED
  * @param  none
  * @retval none
  * @note   lights red LED and passes pointer to light green LED
  */
void red_led(){
  start_device_chain( DEV_LED_RED, 500, &green_led);
/**
  * @brief  Kick off a 500mS green LED with a call to the blue LED
  * @param  none
  * @retval none
  * @note   lights green LED and passes pointer to light blue LED
  */
}
void green_led(){
  start_device( DEV_LED_GREEN, 500);
  start_device( DEV_BUZZER, 500);
}
/** 
  * @brief  Handle the LEDs,beeper and power in turning off the system
  * @param  none
  * @retval success. Always success.
  * @note   Requirements indicate that from any state, 
  *         we need to "rainbow" the LEDs and sound the beeper for 500mS
  *         and power everything down.
  */
uint32_t  processOff( void){
  //  Powering off from ANY state - need to "rainbow" the LEDs...
  start_device( DEV_LED_RED, 500);
  //  power off the TEC - PWM max, TEC_enable off
  start_device( DEV_LED_GREEN, 500);
  //  power off the fan - Fan_enable off
  start_device( DEV_LED_BLUE, 500);
  //  finish up
  return 0;
}
uint32_t processOn( void){
  blink_device( DEV_LED_GREEN, 500, 5000);
  start_device( DEV_BUZZER, 500);
  //  finish up
  
  return 0;
}
/**
  * @brief  State machine for controlling the cooler
  * @param  Event - cooling machine event (power, temp, buttons)
  * @retval State to which we've transitioned
  * @note   Based on flow diagram CS_MasterState.jpg
  */
cooling_state_t processEvent( cooling_event_t event){
  rtc_read_clock( RTC_ADDR, &localTime);
  log_error( stateStr[ getState()]);
  log_error( eventStr[ event]);
  if( CE_ButtonTap == event){
      //  Done from any state save OFF...
    if( ST_IDLE == getState()){
      start_device( DEV_LED_GREEN, t_visible);
      start_device( DEV_BUZZER, t_BEEPidle);
    }
    else if(( ST_COOL == getState())|| (ST_REFRACT == getState() ||ST_PASSIVE == getState())){
      start_device( DEV_LED_BLUE, t_visible);
      start_device( DEV_BUZZER, t_BEEPcool);
    }    
  }
  else if( CE_V_Low == event){
  }
  else if( CE_V_Critical == event){
  }
  switch( getState()){
    case ST_OFF:
      if( CE_PowerOn == event){
        //  test and get ready
      }
      else{
        log_string(localTime.year, localTime.month, localTime.date, localTime.hrs, localTime.mins,
              localTime.secs, localTime.hund,"invalid event in ST_OFF");
      }
      setState( ST_IDLE);
      break;
    case ST_IDLE:
      if( CE_PowerOff == event){
        //  cycle lights and power down
        processOff();
        setState( ST_OFF);
      }
      else if( CE_CoolOn == event){
        //  cycle lights and enable
        start_device( DEV_LED_BLUE, 500);
        
        setState( ST_COOL);
      }
      else{
        log_string(localTime.year, localTime.month, localTime.date, localTime.hrs, localTime.mins,
              localTime.secs, localTime.hund,"invalid event in ST_IDLE");
      }
      break;
    case ST_COOL:
    case ST_PASSIVE:
      if( CE_PowerOff == event){
        //  cycle lights and power down
        processOff();
        setState( ST_OFF);
      }
      else if( CE_CoolOn == event){
        //  switch to Refractory cycle
        setState( ST_REFRACT);
      }
      else{
        log_string(localTime.year, localTime.month, localTime.date, localTime.hrs, localTime.mins,
        localTime.secs, localTime.hund,"invalid event in ST_COOL");
      }
      break;
      
    case ST_REFRACT:
      if( CE_PowerOff == event){
        //  cycle lights and power down
        processOff();
        setState( ST_OFF);
      }
      else if(CE_Refract == event){
        //  switch to Cooling cycle
        setState( ST_COOL);
      }
      else if( CE_CoolOff == event){
        //  switch to Idle
        setState( ST_IDLE);
      }
      break;
    case ST_ERROR:
      //  Cycle lights and bitch, then power down
      processOff();
      setState( ST_OFF);
      break;
    default:
       break;

  }
  return getState();
}


/* [] END OF FILE */
