/**
  ******************************************************************************
  * @file    	timebase.c
  * @brief   	Maintain time base variables and counters
  *	@author		John Griswold
  *
  ******************************************************************************
  *
  *
  * COPYRIGHT(c) 2018 Vivonics, Inc.
  *
  */
  
  
#include  "timebase.h"
#include  "states.h"

extern active_counter_t Devices[ NUM_DEVICES];
void *  FNON[ NUM_DEVICES] = {  &buzzer_on, 
                                &LED_B_ON, 
                                &LED_R_ON, 
                                &LED_G_ON
};
void *  FNOFF[ NUM_DEVICES] = { &buzzer_off,                                 
                                &LED_B_OFF, 
                                &LED_R_OFF, 
                                &LED_G_OFF
};

static int64_t timeBase = 0;
cooling_state_t machineState;

/**
  * @note Timer is set up with 1MHz clock and a 1000 divider
  * @note 100 second timing test yielded 99,888 (99751, 100069) interrupts which is well
  * @note within the margin of error for my mouse handling while watching a 
  * @note stopwatch.
  * @note - Still need to go over this with a scope...
  */

int16_t logcounter = 1000;
int16_t logflag = 0;
  
CY_ISR( Timer_ISR){
  timeBase++;
  Timer_ClearInterrupt(Timer_INTR_MASK_TC);
}
  
uint64_t now( void){
  return timeBase;
}

  
 int32_t  init_timebase( void){
  machineState = ST_OFF;
  ISR_Counter_StartEx( Timer_ISR);
  for( int i = 0; i < NUM_DEVICES; i++){
    Devices[ i].counter_state = DEV_OFF;
    Devices[ i].stop_time = 0;
    Devices[ i].stop_time = 0;
    Devices[ i].ms_interval_preload = 0;
    Devices[ i].pfnOff = FNOFF[ i];
    Devices[ i].pfnOn  = FNON[ i];
  }
  return 0;
}
/**
  * @brief  timebase_tick processes the timers associated with devices
  * @param  None
  * @retval 0 if success
  * @note This routine needs to account for blinking and solid lit LEDs for a period of time (in mS...)
  * @note This routine also manages the buzzer
  *
  * @note - devices can be off, on, blinking(on), blinking( off)
  * @note - devices get turned off if current time > stop_time
  * @note - blinking devices toggle On and off if current time > toggle_time
  *
*/
int32_t timebase_tick( void){
  int64_t current_time = now();
  
  for( int i = 0; i < NUM_DEVICES; i++){
    if( current_time > Devices[i].stop_time ){  //  expired timer...
      kill_device( i);
    }
    else{
      switch( Devices[ i].counter_state){
        
        case  DEV_OFF:
          Devices[ i].stop_time = 0;                  //  Shouldn't really get here - just set stop_time as a flag
          break;                                      //  And yes, it's redundant
          
        case  DEV_ON:                                 //  Device is on. If timeout, just turn it off.
          if( current_time > Devices[ i].stop_time){
            kill_device( i);
            if( Devices[ i].pfnNext){
              Devices[ i].pfnNext();
            }
          }
          break;
          
        case  DEV_BLINK_ON:                             //  Device is blinking, but on...
          if( current_time >  Devices[ i].toggle_time){ //  But done with this interval - switch
            Devices[ i].toggle_time += Devices[ i].ms_interval_preload;
            Devices[ i].counter_state = DEV_BLINK_OFF;
            Devices[ i].pfnOff();
          }
          break;
          
          
        case  DEV_BLINK_OFF:                            //  Device is blinking, but off...
          if( current_time >  Devices[ i].toggle_time){ //  But done with this interval - switch
            Devices[ i].toggle_time += Devices[ i].ms_interval_preload;
            Devices[ i].counter_state = DEV_BLINK_ON;
            Devices[ i].pfnOn();
          }
          break;
        
        case DEV_CONTINUOUS:
          break;
          
        default:
          log_error("Default state in timebase_tick()");
          break;
      }
    }
  }
  return 0;
}

/* [] END OF FILE */
